#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2019 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import argparse
import fnmatch
import logging
import os
import os.path
import subprocess
import sys
import zipfile

logging.basicConfig(format='%(message)s')

# Flavors of ART APEX package.
FLAVOR_RELEASE = 'release'
FLAVOR_DEBUG = 'debug'
FLAVOR_TESTING = 'testing'
FLAVOR_AUTO = 'auto'
FLAVORS_ALL = [FLAVOR_RELEASE, FLAVOR_DEBUG, FLAVOR_TESTING, FLAVOR_AUTO]

# Bitness options for APEX package
BITNESS_32 = '32'
BITNESS_64 = '64'
BITNESS_MULTILIB = 'multilib'
BITNESS_AUTO = 'auto'
BITNESS_ALL = [BITNESS_32, BITNESS_64, BITNESS_MULTILIB, BITNESS_AUTO]

# Architectures supported by APEX packages.
ARCHS = ["arm", "arm64", "x86", "x86_64"]

# Directory containing ART tests within an ART APEX (if the package includes
# any). ART test executables are installed in `bin/art/<arch>`. Segregating
# tests by architecture is useful on devices supporting more than one
# architecture, as it permits testing all of them using a single ART APEX
# package.
ART_TEST_DIR = 'bin/art'


# Test if a given variable is set to a string "true".
def isEnvTrue(var):
  return var in os.environ and os.environ[var] == 'true'


class FSObject:
  def __init__(self, name, is_dir, is_exec, is_symlink, size):
    self.name = name
    self.is_dir = is_dir
    self.is_exec = is_exec
    self.is_symlink = is_symlink
    self.size = size

  def __str__(self):
    return '%s(dir=%r,exec=%r,symlink=%r,size=%d)' \
             % (self.name, self.is_dir, self.is_exec, self.is_symlink, self.size)


class TargetApexProvider:
  def __init__(self, apex, tmpdir, debugfs):
    self._tmpdir = tmpdir
    self._debugfs = debugfs
    self._folder_cache = {}
    self._payload = os.path.join(self._tmpdir, 'apex_payload.img')
    # Extract payload to tmpdir.
    apex_zip = zipfile.ZipFile(apex)
    apex_zip.extract('apex_payload.img', tmpdir)

  def __del__(self):
    # Delete temps.
    if os.path.exists(self._payload):
      os.remove(self._payload)

  def get(self, path):
    apex_dir, name = os.path.split(path)
    if not apex_dir:
      apex_dir = '.'
    apex_map = self.read_dir(apex_dir)
    return apex_map[name] if name in apex_map else None

  def read_dir(self, apex_dir):
    if apex_dir in self._folder_cache:
      return self._folder_cache[apex_dir]
    # Cannot use check_output as it will annoy with stderr.
    process = subprocess.Popen([self._debugfs, '-R', 'ls -l -p %s' % apex_dir, self._payload],
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                               universal_newlines=True)
    stdout, _ = process.communicate()
    res = str(stdout)
    apex_map = {}
    # Debugfs output looks like this:
    #   debugfs 1.44.4 (18-Aug-2018)
    #   /12/040755/0/2000/.//
    #   /2/040755/1000/1000/..//
    #   /13/100755/0/2000/dalvikvm32/28456/
    #   /14/100755/0/2000/dexoptanalyzer/20396/
    #   /15/100755/0/2000/linker/1152724/
    #   /16/100755/0/2000/dex2oat/563508/
    #   /17/100755/0/2000/linker64/1605424/
    #   /18/100755/0/2000/profman/85304/
    #   /19/100755/0/2000/dalvikvm64/28576/
    #    |     |   |   |       |        |
    #    |     |   |   #- gid  #- name  #- size
    #    |     |   #- uid
    #    |     #- type and permission bits
    #    #- inode nr (?)
    #
    # Note: could break just on '/' to avoid names with newlines.
    for line in res.split("\n"):
      if not line:
        continue
      comps = line.split('/')
      if len(comps) != 8:
        logging.warning('Could not break and parse line \'%s\'', line)
        continue
      bits = comps[2]
      name = comps[5]
      size_str = comps[6]
      # Use a negative value as an indicator of undefined/unknown size.
      size = int(size_str) if size_str != '' else -1
      if len(bits) != 6:
        logging.warning('Dont understand bits \'%s\'', bits)
        continue
      is_dir = bits[1] == '4'

      def is_exec_bit(ch):
        return int(ch) & 1 == 1

      is_exec = is_exec_bit(bits[3]) and is_exec_bit(bits[4]) and is_exec_bit(bits[5])
      is_symlink = bits[1] == '2'
      apex_map[name] = FSObject(name, is_dir, is_exec, is_symlink, size)
    self._folder_cache[apex_dir] = apex_map
    return apex_map


class TargetFlattenedApexProvider:
  def __init__(self, apex):
    self._folder_cache = {}
    self._apex = apex

  def get(self, path):
    apex_dir, name = os.path.split(path)
    if not apex_dir:
      apex_dir = '.'
    apex_map = self.read_dir(apex_dir)
    return apex_map[name] if name in apex_map else None

  def read_dir(self, apex_dir):
    if apex_dir in self._folder_cache:
      return self._folder_cache[apex_dir]
    apex_map = {}
    dirname = os.path.join(self._apex, apex_dir)
    if os.path.exists(dirname):
      for basename in os.listdir(dirname):
        filepath = os.path.join(dirname, basename)
        is_dir = os.path.isdir(filepath)
        is_exec = os.access(filepath, os.X_OK)
        is_symlink = os.path.islink(filepath)
        if is_symlink:
          # Report the length of the symlink's target's path as file size, like `ls`.
          size = len(os.readlink(filepath))
        else:
          size = os.path.getsize(filepath)
        apex_map[basename] = FSObject(basename, is_dir, is_exec, is_symlink, size)
    self._folder_cache[apex_dir] = apex_map
    return apex_map


class HostApexProvider:
  def __init__(self, apex, tmpdir):
    self._tmpdir = tmpdir
    self._folder_cache = {}
    self._payload = os.path.join(self._tmpdir, 'apex_payload.zip')
    # Extract payload to tmpdir.
    apex_zip = zipfile.ZipFile(apex)
    apex_zip.extract('apex_payload.zip', tmpdir)

  def __del__(self):
    # Delete temps.
    if os.path.exists(self._payload):
      os.remove(self._payload)

  def get(self, path):
    apex_dir, name = os.path.split(path)
    if not apex_dir:
      apex_dir = ''
    apex_map = self.read_dir(apex_dir)
    return apex_map[name] if name in apex_map else None

  def read_dir(self, apex_dir):
    if apex_dir in self._folder_cache:
      return self._folder_cache[apex_dir]
    if not self._folder_cache:
      self.parse_zip()
    if apex_dir in self._folder_cache:
      return self._folder_cache[apex_dir]
    return {}

  def parse_zip(self):
    apex_zip = zipfile.ZipFile(self._payload)
    infos = apex_zip.infolist()
    for zipinfo in infos:
      path = zipinfo.filename

      # Assume no empty file is stored.
      assert path

      def get_octal(val, index):
        return (val >> (index * 3)) & 0x7

      def bits_is_exec(val):
        # TODO: Enforce group/other, too?
        return get_octal(val, 2) & 1 == 1

      is_zipinfo = True
      while path:
        apex_dir, base = os.path.split(path)
        # TODO: If directories are stored, base will be empty.

        if apex_dir not in self._folder_cache:
          self._folder_cache[apex_dir] = {}
        dir_map = self._folder_cache[apex_dir]
        if base not in dir_map:
          if is_zipinfo:
            bits = (zipinfo.external_attr >> 16) & 0xFFFF
            is_dir = get_octal(bits, 4) == 4
            is_symlink = get_octal(bits, 4) == 2
            is_exec = bits_is_exec(bits)
            size = zipinfo.file_size
          else:
            is_exec = False  # Seems we can't get this easily?
            is_symlink = False
            is_dir = True
            # Use a negative value as an indicator of undefined/unknown size.
            size = -1
          dir_map[base] = FSObject(base, is_dir, is_exec, is_symlink, size)
        is_zipinfo = False
        path = apex_dir


# DO NOT USE DIRECTLY! This is an "abstract" base class.
class Checker:
  def __init__(self, provider):
    self._provider = provider
    self._errors = 0
    self._expected_file_globs = set()

  def fail(self, msg, *fail_args):
    self._errors += 1
    logging.error(msg, *fail_args)

  def error_count(self):
    return self._errors

  def reset_errors(self):
    self._errors = 0

  def is_file(self, path):
    fs_object = self._provider.get(path)
    if fs_object is None:
      return False, 'Could not find %s'
    if fs_object.is_dir:
      return False, '%s is a directory'
    return True, ''

  def is_dir(self, path):
    fs_object = self._provider.get(path)
    if fs_object is None:
      return False, 'Could not find %s'
    if not fs_object.is_dir:
      return False, '%s is not a directory'
    return True, ''

  def check_file(self, path):
    ok, msg = self.is_file(path)
    if not ok:
      self.fail(msg, path)
    self._expected_file_globs.add(path)
    return ok

  def check_executable(self, filename):
    path = 'bin/%s' % filename
    if not self.check_file(path):
      return
    if not self._provider.get(path).is_exec:
      self.fail('%s is not executable', path)

  def check_executable_symlink(self, filename):
    path = 'bin/%s' % filename
    fs_object = self._provider.get(path)
    if fs_object is None:
      self.fail('Could not find %s', path)
      return
    if fs_object.is_dir:
      self.fail('%s is a directory', path)
      return
    if not fs_object.is_symlink:
      self.fail('%s is not a symlink', path)
    self._expected_file_globs.add(path)

  def arch_dirs_for_path(self, path):
    # Look for target-specific subdirectories for the given directory path.
    # This is needed because the list of build targets is not propagated
    # to this script.
    #
    # TODO(b/123602136): Pass build target information to this script and fix
    # all places where this function in used (or similar workarounds).
    dirs = []
    for arch in ARCHS:
      dir = '%s/%s' % (path, arch)
      found, _ = self.is_dir(dir)
      if found:
        dirs.append(dir)
    return dirs

  def check_art_test_executable(self, filename):
    dirs = self.arch_dirs_for_path(ART_TEST_DIR)
    if not dirs:
      self.fail('ART test binary missing: %s', filename)
    for dir in dirs:
      test_path = '%s/%s' % (dir, filename)
      self._expected_file_globs.add(test_path)
      if not self._provider.get(test_path).is_exec:
        self.fail('%s is not executable', test_path)

  def check_single_library(self, filename):
    lib_path = 'lib/%s' % filename
    lib64_path = 'lib64/%s' % filename
    lib_is_file, _ = self.is_file(lib_path)
    if lib_is_file:
      self._expected_file_globs.add(lib_path)
    lib64_is_file, _ = self.is_file(lib64_path)
    if lib64_is_file:
      self._expected_file_globs.add(lib64_path)
    if not lib_is_file and not lib64_is_file:
      self.fail('Library missing: %s', filename)

  def check_dexpreopt(self, basename):
    dirs = self.arch_dirs_for_path('javalib')
    for dir in dirs:
      for ext in ['art', 'oat', 'vdex']:
        self.check_file('%s/%s.%s' % (dir, basename, ext))

  def check_java_library(self, basename):
    return self.check_file('javalib/%s.jar' % basename)

  def ignore_path(self, path_glob):
    self._expected_file_globs.add(path_glob)

  def check_optional_art_test_executable(self, filename):
    for arch in ARCHS:
      self.ignore_path('%s/%s/%s' % (ART_TEST_DIR, arch, filename))

  def check_no_superfluous_files(self, dir_path):
    paths = []
    for name in sorted(self._provider.read_dir(dir_path).keys()):
      if name not in ('.', '..'):
        paths.append(os.path.join(dir_path, name))
    expected_paths = set()
    dir_prefix = dir_path + '/'
    for path_glob in self._expected_file_globs:
      expected_paths |= set(fnmatch.filter(paths, path_glob))
      # If there are globs in subdirectories of dir_path we want to match their
      # path segments at this directory level.
      if path_glob.startswith(dir_prefix):
        subpath = path_glob[len(dir_prefix):]
        subpath_first_segment, _, _ = subpath.partition('/')
        expected_paths |= set(fnmatch.filter(paths, dir_prefix + subpath_first_segment))
    for unexpected_path in set(paths) - expected_paths:
      self.fail('Unexpected file \'%s\'', unexpected_path)

  # Just here for docs purposes, even if it isn't good Python style.

  def check_symlinked_multilib_executable(self, filename):
    """Check bin/filename32, and/or bin/filename64, with symlink bin/filename."""
    raise NotImplementedError

  def check_symlinked_first_executable(self, filename):
    """Check bin/filename32, and/or bin/filename64, with symlink bin/filename."""
    raise NotImplementedError

  def check_multilib_executable(self, filename):
    """Check bin/filename for 32 bit, and/or bin/filename64."""
    raise NotImplementedError

  def check_first_executable(self, filename):
    """Check bin/filename for 32 bit, and/or bin/filename64."""
    raise NotImplementedError

  def check_native_library(self, basename):
    """Check lib/basename.so, and/or lib64/basename.so."""
    raise NotImplementedError

  def check_optional_native_library(self, basename_glob):
    """Allow lib/basename.so and/or lib64/basename.so to exist."""
    raise NotImplementedError

  def check_prefer64_library(self, basename):
    """Check lib64/basename.so, or lib/basename.so on 32 bit only."""
    raise NotImplementedError


class Arch32Checker(Checker):
  def check_symlinked_multilib_executable(self, filename):
    self.check_executable('%s32' % filename)
    self.check_executable_symlink(filename)

  def check_symlinked_first_executable(self, filename):
    self.check_executable('%s32' % filename)
    self.check_executable_symlink(filename)

  def check_multilib_executable(self, filename):
    self.check_executable('%s32' % filename)

  def check_first_executable(self, filename):
    self.check_executable('%s32' % filename)

  def check_native_library(self, basename):
    # TODO: Use $TARGET_ARCH (e.g. check whether it is "arm" or "arm64") to improve
    # the precision of this test?
    self.check_file('lib/%s.so' % basename)

  def check_optional_native_library(self, basename_glob):
    self.ignore_path('lib/%s.so' % basename_glob)

  def check_prefer64_library(self, basename):
    self.check_native_library(basename)


class Arch64Checker(Checker):
  def check_symlinked_multilib_executable(self, filename):
    self.check_executable('%s64' % filename)
    self.check_executable_symlink(filename)

  def check_symlinked_first_executable(self, filename):
    self.check_executable('%s64' % filename)
    self.check_executable_symlink(filename)

  def check_multilib_executable(self, filename):
    self.check_executable('%s64' % filename)

  def check_first_executable(self, filename):
    self.check_executable('%s64' % filename)

  def check_native_library(self, basename):
    # TODO: Use $TARGET_ARCH (e.g. check whether it is "arm" or "arm64") to improve
    # the precision of this test?
    self.check_file('lib64/%s.so' % basename)

  def check_optional_native_library(self, basename_glob):
    self.ignore_path('lib64/%s.so' % basename_glob)

  def check_prefer64_library(self, basename):
    self.check_native_library(basename)


class MultilibChecker(Checker):
  def check_symlinked_multilib_executable(self, filename):
    self.check_executable('%s32' % filename)
    self.check_executable('%s64' % filename)
    self.check_executable_symlink(filename)

  def check_symlinked_first_executable(self, filename):
    self.check_executable('%s64' % filename)
    self.check_executable_symlink(filename)

  def check_multilib_executable(self, filename):
    self.check_executable('%s64' % filename)
    self.check_executable('%s32' % filename)

  def check_first_executable(self, filename):
    self.check_executable('%s64' % filename)

  def check_native_library(self, basename):
    # TODO: Use $TARGET_ARCH (e.g. check whether it is "arm" or "arm64") to improve
    # the precision of this test?
    self.check_file('lib/%s.so' % basename)
    self.check_file('lib64/%s.so' % basename)

  def check_optional_native_library(self, basename_glob):
    self.ignore_path('lib/%s.so' % basename_glob)
    self.ignore_path('lib64/%s.so' % basename_glob)

  def check_prefer64_library(self, basename):
    self.check_file('lib64/%s.so' % basename)


class ReleaseChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'Release Checker'

  def run(self):
    # Check the Protocol Buffers APEX manifest.
    self._checker.check_file('apex_manifest.pb')

    # Check binaries for ART.
    self._checker.check_first_executable('dex2oat')
    self._checker.check_executable('dexdump')
    self._checker.check_executable('dexlist')
    self._checker.check_executable('dexoptanalyzer')
    self._checker.check_executable('profman')
    self._checker.check_symlinked_multilib_executable('dalvikvm')

    # Check exported libraries for ART.
    self._checker.check_native_library('libdexfile_external')
    self._checker.check_native_library('libnativebridge')
    self._checker.check_native_library('libnativehelper')
    self._checker.check_native_library('libnativeloader')

    # Check internal libraries for ART.
    self._checker.check_native_library('libadbconnection')
    self._checker.check_native_library('libart')
    self._checker.check_native_library('libart-compiler')
    self._checker.check_native_library('libart-dexlayout')
    self._checker.check_native_library('libart-disassembler')
    self._checker.check_native_library('libartbase')
    self._checker.check_native_library('libartpalette')
    self._checker.check_native_library('libdexfile')
    self._checker.check_native_library('libdexfile_support')
    self._checker.check_native_library('libopenjdkjvm')
    self._checker.check_native_library('libopenjdkjvmti')
    self._checker.check_native_library('libprofile')
    self._checker.check_native_library('libsigchain')

    # Check java libraries for Managed Core Library.
    self._checker.check_java_library('apache-xml')
    self._checker.check_java_library('bouncycastle')
    self._checker.check_java_library('core-icu4j')
    self._checker.check_java_library('core-libart')
    self._checker.check_java_library('core-oj')
    self._checker.check_java_library('okhttp')
    if isEnvTrue('EMMA_INSTRUMENT_FRAMEWORK'):
      # In coverage builds jacoco is added to the list of ART apex jars.
      self._checker.check_java_library('jacocoagent')

    # Check internal native libraries for Managed Core Library.
    self._checker.check_native_library('libjavacore')
    self._checker.check_native_library('libopenjdk')

    # Check internal native library dependencies.
    #
    # Any internal dependency not listed here will cause a failure in
    # NoSuperfluousLibrariesChecker. Internal dependencies are generally just
    # implementation details, but in the release package we want to track them
    # because a) they add to the package size and the RAM usage (in particular
    # if the library is also present in /system or another APEX and hence might
    # get loaded twice through linker namespace separation), and b) we need to
    # catch invalid dependencies on /system or other APEXes that should go
    # through an exported library with stubs (b/128708192 tracks implementing a
    # better approach for that).
    self._checker.check_native_library('libbacktrace')
    self._checker.check_native_library('libbase')
    self._checker.check_native_library('libc++')
    self._checker.check_native_library('libdt_fd_forward')
    self._checker.check_native_library('libdt_socket')
    self._checker.check_native_library('libjdwp')
    self._checker.check_native_library('liblzma')
    self._checker.check_native_library('libnpt')
    self._checker.check_native_library('libunwindstack')
    self._checker.check_native_library('libziparchive')
    self._checker.check_optional_native_library('libvixl')  # Only on ARM/ARM64

    # Allow extra dependencies that appear in ASAN builds.
    self._checker.check_optional_native_library('libclang_rt.asan*')
    self._checker.check_optional_native_library('libclang_rt.hwasan*')
    self._checker.check_optional_native_library('libclang_rt.ubsan*')

    # Check dexpreopt files for libcore bootclasspath jars.
    self._checker.check_dexpreopt('boot')
    self._checker.check_dexpreopt('boot-apache-xml')
    self._checker.check_dexpreopt('boot-bouncycastle')
    self._checker.check_dexpreopt('boot-core-icu4j')
    self._checker.check_dexpreopt('boot-core-libart')
    self._checker.check_dexpreopt('boot-okhttp')
    if isEnvTrue('EMMA_INSTRUMENT_FRAMEWORK'):
      # In coverage builds the ART boot image includes jacoco.
      self._checker.check_dexpreopt('boot-jacocoagent')

class ReleaseTargetChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'Release (Target) Checker'

  def run(self):
    # We don't check for the presence of the JSON APEX manifest (file
    # `apex_manifest.json`, only present in target APEXes), as it is only
    # included for compatibility reasons with Android Q and will likely be
    # removed in Android R.

    # Check binaries for ART.
    self._checker.check_executable('oatdump')
    self._checker.check_multilib_executable('dex2oat')

    # Check internal libraries for ART.
    self._checker.check_prefer64_library('libart-disassembler')
    self._checker.check_native_library('libperfetto_hprof')

    # Check exported native libraries for Managed Core Library.
    self._checker.check_native_library('libandroidicu')
    self._checker.check_native_library('libandroidio')

    # Check internal native library dependencies.
    self._checker.check_native_library('libcrypto')
    self._checker.check_native_library('libexpat')
    self._checker.check_native_library('libicui18n')
    self._checker.check_native_library('libicuuc')
    self._checker.check_native_library('libicu_jni')
    self._checker.check_native_library('libpac')
    self._checker.check_native_library('libz')

    # TODO(b/139046641): Fix proper 2nd arch checks. For now, just ignore these
    # directories.
    self._checker.ignore_path('bin/arm')
    self._checker.ignore_path('lib/arm')
    self._checker.ignore_path('lib64/arm')


class ReleaseHostChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'Release (Host) Checker'

  def run(self):
    # Check binaries for ART.
    self._checker.check_executable('hprof-conv')
    self._checker.check_symlinked_first_executable('dex2oatd')
    self._checker.check_symlinked_first_executable('dex2oat')

    # Check exported native libraries for Managed Core Library.
    self._checker.check_native_library('libandroidicu-host')
    self._checker.check_native_library('libandroidio')

    # Check internal libraries for Managed Core Library.
    self._checker.check_native_library('libexpat-host')
    self._checker.check_native_library('libicui18n-host')
    self._checker.check_native_library('libicuuc-host')
    self._checker.check_native_library('libicu_jni')
    self._checker.check_native_library('libz-host')


class DebugChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'Debug Checker'

  def run(self):
    # Check binaries for ART.
    self._checker.check_executable('dexdiag')
    self._checker.check_executable('dexanalyze')
    self._checker.check_executable('dexlayout')
    self._checker.check_symlinked_multilib_executable('imgdiag')

    # Check debug binaries for ART.
    self._checker.check_executable('dexlayoutd')
    self._checker.check_executable('dexoptanalyzerd')
    self._checker.check_symlinked_multilib_executable('imgdiagd')
    self._checker.check_executable('profmand')

    # Check internal libraries for ART.
    self._checker.check_native_library('libadbconnectiond')
    self._checker.check_native_library('libart-disassembler')
    self._checker.check_native_library('libartbased')
    self._checker.check_native_library('libartd')
    self._checker.check_native_library('libartd-compiler')
    self._checker.check_native_library('libartd-dexlayout')
    self._checker.check_native_library('libartd-disassembler')
    self._checker.check_native_library('libdexfiled')
    self._checker.check_native_library('libopenjdkjvmd')
    self._checker.check_native_library('libopenjdkjvmtid')
    self._checker.check_native_library('libprofiled')

    # Check internal libraries for Managed Core Library.
    self._checker.check_native_library('libopenjdkd')


class DebugTargetChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'Debug (Target) Checker'

  def run(self):
    # Check ART debug binaries.
    self._checker.check_multilib_executable('dex2oatd')
    self._checker.check_multilib_executable('dex2oat')
    self._checker.check_executable('oatdumpd')

    # Check ART internal libraries.
    self._checker.check_native_library('libdexfiled_external')
    self._checker.check_native_library('libperfetto_hprofd')

    # Check internal native library dependencies.
    #
    # Like in the release package, we check that we don't get other dependencies
    # besides those listed here. In this case the concern is not bloat, but
    # rather that we don't get behavioural differences between user (release)
    # and userdebug/eng builds, which could happen if the debug package has
    # duplicate library instances where releases don't. In other words, it's
    # uncontroversial to add debug-only dependencies, as long as they don't make
    # assumptions on having a single global state (ideally they should have
    # double_loadable:true, cf. go/double_loadable). Also, like in the release
    # package we need to look out for dependencies that should go through
    # exported library stubs (until b/128708192 is fixed).
    self._checker.check_optional_native_library('libvixld')  # Only on ARM/ARM64
    self._checker.check_prefer64_library('libmeminfo')
    self._checker.check_prefer64_library('libprocinfo')


class TestingTargetChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'Testing (Target) Checker'

  def run(self):
    # Check cmdline tests.
    self._checker.check_optional_art_test_executable('cmdline_parser_test')

    # Check compiler tests.
    self._checker.check_art_test_executable('atomic_dex_ref_map_test')
    self._checker.check_art_test_executable('bounds_check_elimination_test')
    self._checker.check_art_test_executable('codegen_test')
    self._checker.check_art_test_executable('compiled_method_storage_test')
    self._checker.check_art_test_executable('data_type_test')
    self._checker.check_art_test_executable('dedupe_set_test')
    self._checker.check_art_test_executable('dominator_test')
    self._checker.check_art_test_executable('dwarf_test')
    self._checker.check_art_test_executable('exception_test')
    self._checker.check_art_test_executable('find_loops_test')
    self._checker.check_art_test_executable('graph_checker_test')
    self._checker.check_art_test_executable('graph_test')
    self._checker.check_art_test_executable('gvn_test')
    self._checker.check_art_test_executable('induction_var_analysis_test')
    self._checker.check_art_test_executable('induction_var_range_test')
    self._checker.check_art_test_executable('jni_cfi_test')
    self._checker.check_art_test_executable('jni_compiler_test')
    self._checker.check_art_test_executable('licm_test')
    self._checker.check_art_test_executable('linker_patch_test')
    self._checker.check_art_test_executable('live_interval_test')
    self._checker.check_art_test_executable('load_store_analysis_test')
    self._checker.check_art_test_executable('load_store_elimination_test')
    self._checker.check_art_test_executable('loop_optimization_test')
    self._checker.check_art_test_executable('nodes_test')
    self._checker.check_art_test_executable('nodes_vector_test')
    self._checker.check_art_test_executable('optimizing_cfi_test')
    self._checker.check_art_test_executable('output_stream_test')
    self._checker.check_art_test_executable('parallel_move_test')
    self._checker.check_art_test_executable('pretty_printer_test')
    self._checker.check_art_test_executable('reference_type_propagation_test')
    self._checker.check_art_test_executable('scheduler_test')
    self._checker.check_art_test_executable('select_generator_test')
    self._checker.check_art_test_executable('side_effects_test')
    self._checker.check_art_test_executable('src_map_elem_test')
    self._checker.check_art_test_executable('ssa_liveness_analysis_test')
    self._checker.check_art_test_executable('ssa_test')
    self._checker.check_art_test_executable('stack_map_test')
    self._checker.check_art_test_executable('superblock_cloner_test')
    self._checker.check_art_test_executable('suspend_check_test')
    self._checker.check_art_test_executable('swap_space_test')
    # These tests depend on a specific code generator and are conditionally included.
    self._checker.check_optional_art_test_executable('constant_folding_test')
    self._checker.check_optional_art_test_executable('dead_code_elimination_test')
    self._checker.check_optional_art_test_executable('linearize_test')
    self._checker.check_optional_art_test_executable('live_ranges_test')
    self._checker.check_optional_art_test_executable('liveness_test')
    self._checker.check_optional_art_test_executable('managed_register_arm64_test')
    self._checker.check_optional_art_test_executable('managed_register_arm_test')
    self._checker.check_optional_art_test_executable('managed_register_x86_64_test')
    self._checker.check_optional_art_test_executable('managed_register_x86_test')
    self._checker.check_optional_art_test_executable('register_allocator_test')

    # Check dex2oat tests.
    self._checker.check_art_test_executable('compiler_driver_test')
    self._checker.check_art_test_executable('dex2oat_image_test')
    self._checker.check_art_test_executable('dex2oat_test')
    self._checker.check_art_test_executable('dex2oat_vdex_test')
    self._checker.check_art_test_executable('dex_to_dex_decompiler_test')
    self._checker.check_art_test_executable('elf_writer_test')
    self._checker.check_art_test_executable('image_test')
    self._checker.check_art_test_executable('image_write_read_test')
    self._checker.check_art_test_executable('index_bss_mapping_encoder_test')
    self._checker.check_art_test_executable('multi_oat_relative_patcher_test')
    self._checker.check_art_test_executable('oat_writer_test')
    self._checker.check_art_test_executable('verifier_deps_test')
    # These tests depend on a specific code generator and are conditionally included.
    self._checker.check_optional_art_test_executable('relative_patcher_arm64_test')
    self._checker.check_optional_art_test_executable('relative_patcher_thumb2_test')
    self._checker.check_optional_art_test_executable('relative_patcher_x86_64_test')
    self._checker.check_optional_art_test_executable('relative_patcher_x86_test')

    # Check dexanalyze tests.
    self._checker.check_optional_art_test_executable('dexanalyze_test')

    # Check dexdiag tests.
    self._checker.check_optional_art_test_executable('dexdiag_test')

    # Check dexdump tests.
    self._checker.check_art_test_executable('dexdump_test')

    # Check dexlayout tests.
    self._checker.check_optional_art_test_executable('dexlayout_test')

    # Check dexlist tests.
    self._checker.check_art_test_executable('dexlist_test')

    # Check dexoptanalyzer tests.
    self._checker.check_art_test_executable('dexoptanalyzer_test')

    # Check imgdiag tests.
    self._checker.check_art_test_executable('imgdiag_test')

    # Check libartbase tests.
    self._checker.check_art_test_executable('arena_allocator_test')
    self._checker.check_art_test_executable('bit_field_test')
    self._checker.check_art_test_executable('bit_memory_region_test')
    self._checker.check_art_test_executable('bit_string_test')
    self._checker.check_art_test_executable('bit_struct_test')
    self._checker.check_art_test_executable('bit_table_test')
    self._checker.check_art_test_executable('bit_utils_test')
    self._checker.check_art_test_executable('bit_vector_test')
    self._checker.check_art_test_executable('fd_file_test')
    self._checker.check_art_test_executable('file_utils_test')
    self._checker.check_art_test_executable('hash_set_test')
    self._checker.check_art_test_executable('hex_dump_test')
    self._checker.check_art_test_executable('histogram_test')
    self._checker.check_art_test_executable('indenter_test')
    self._checker.check_art_test_executable('instruction_set_test')
    self._checker.check_art_test_executable('intrusive_forward_list_test')
    self._checker.check_art_test_executable('leb128_test')
    self._checker.check_art_test_executable('logging_test')
    self._checker.check_art_test_executable('mem_map_test')
    self._checker.check_art_test_executable('membarrier_test')
    self._checker.check_art_test_executable('memfd_test')
    self._checker.check_art_test_executable('memory_region_test')
    self._checker.check_art_test_executable('safe_copy_test')
    self._checker.check_art_test_executable('scoped_flock_test')
    self._checker.check_art_test_executable('time_utils_test')
    self._checker.check_art_test_executable('transform_array_ref_test')
    self._checker.check_art_test_executable('transform_iterator_test')
    self._checker.check_art_test_executable('utils_test')
    self._checker.check_art_test_executable('variant_map_test')
    self._checker.check_art_test_executable('zip_archive_test')

    # Check libartpalette tests.
    self._checker.check_art_test_executable('palette_test')

    # Check libdexfile tests.
    self._checker.check_art_test_executable('art_dex_file_loader_test')
    self._checker.check_art_test_executable('art_libdexfile_support_tests')
    self._checker.check_art_test_executable('class_accessor_test')
    self._checker.check_art_test_executable('code_item_accessors_test')
    self._checker.check_art_test_executable('compact_dex_file_test')
    self._checker.check_art_test_executable('compact_offset_table_test')
    self._checker.check_art_test_executable('descriptors_names_test')
    self._checker.check_art_test_executable('dex_file_loader_test')
    self._checker.check_art_test_executable('dex_file_verifier_test')
    self._checker.check_art_test_executable('dex_instruction_test')
    self._checker.check_art_test_executable('primitive_test')
    self._checker.check_art_test_executable('string_reference_test')
    self._checker.check_art_test_executable('test_dex_file_builder_test')
    self._checker.check_art_test_executable('type_lookup_table_test')
    self._checker.check_art_test_executable('utf_test')

    # Check libprofile tests.
    self._checker.check_optional_art_test_executable('profile_boot_info_test')
    self._checker.check_optional_art_test_executable('profile_compilation_info_test')

    # Check oatdump tests.
    self._checker.check_art_test_executable('oatdump_app_test')
    self._checker.check_art_test_executable('oatdump_image_test')
    self._checker.check_art_test_executable('oatdump_test')

    # Check profman tests.
    self._checker.check_art_test_executable('profile_assistant_test')

    # Check runtime compiler tests.
    self._checker.check_art_test_executable('module_exclusion_test')
    self._checker.check_art_test_executable('reflection_test')

    # Check runtime tests.
    self._checker.check_art_test_executable('arch_test')
    self._checker.check_art_test_executable('barrier_test')
    self._checker.check_art_test_executable('card_table_test')
    self._checker.check_art_test_executable('cha_test')
    self._checker.check_art_test_executable('class_linker_test')
    self._checker.check_art_test_executable('class_loader_context_test')
    self._checker.check_art_test_executable('class_table_test')
    self._checker.check_art_test_executable('compiler_filter_test')
    self._checker.check_art_test_executable('dex_cache_test')
    self._checker.check_art_test_executable('dlmalloc_space_random_test')
    self._checker.check_art_test_executable('dlmalloc_space_static_test')
    self._checker.check_art_test_executable('entrypoints_order_test')
    self._checker.check_art_test_executable('exec_utils_test')
    self._checker.check_art_test_executable('gtest_test')
    self._checker.check_art_test_executable('handle_scope_test')
    self._checker.check_art_test_executable('heap_test')
    self._checker.check_art_test_executable('heap_verification_test')
    self._checker.check_art_test_executable('hidden_api_test')
    self._checker.check_art_test_executable('image_space_test')
    self._checker.check_art_test_executable('immune_spaces_test')
    self._checker.check_art_test_executable('imtable_test')
    self._checker.check_art_test_executable('indirect_reference_table_test')
    self._checker.check_art_test_executable('instruction_set_features_arm64_test')
    self._checker.check_art_test_executable('instruction_set_features_arm_test')
    self._checker.check_art_test_executable('instruction_set_features_test')
    self._checker.check_art_test_executable('instruction_set_features_x86_64_test')
    self._checker.check_art_test_executable('instruction_set_features_x86_test')
    self._checker.check_art_test_executable('instrumentation_test')
    self._checker.check_art_test_executable('intern_table_test')
    self._checker.check_art_test_executable('java_vm_ext_test')
    self._checker.check_art_test_executable('jit_memory_region_test')
    self._checker.check_art_test_executable('jni_internal_test')
    self._checker.check_art_test_executable('large_object_space_test')
    self._checker.check_art_test_executable('math_entrypoints_test')
    self._checker.check_art_test_executable('memcmp16_test')
    self._checker.check_art_test_executable('method_handles_test')
    self._checker.check_art_test_executable('method_type_test')
    self._checker.check_art_test_executable('method_verifier_test')
    self._checker.check_art_test_executable('mod_union_table_test')
    self._checker.check_art_test_executable('monitor_pool_test')
    self._checker.check_art_test_executable('monitor_test')
    self._checker.check_art_test_executable('mutex_test')
    self._checker.check_art_test_executable('oat_file_assistant_test')
    self._checker.check_art_test_executable('oat_file_test')
    self._checker.check_art_test_executable('object_test')
    self._checker.check_art_test_executable('parsed_options_test')
    self._checker.check_art_test_executable('prebuilt_tools_test')
    self._checker.check_art_test_executable('profiling_info_test')
    self._checker.check_art_test_executable('profile_saver_test')
    self._checker.check_art_test_executable('proxy_test')
    self._checker.check_art_test_executable('quick_trampoline_entrypoints_test')
    self._checker.check_art_test_executable('reference_queue_test')
    self._checker.check_art_test_executable('reference_table_test')
    self._checker.check_art_test_executable('reg_type_test')
    self._checker.check_art_test_executable('rosalloc_space_random_test')
    self._checker.check_art_test_executable('rosalloc_space_static_test')
    self._checker.check_art_test_executable('runtime_callbacks_test')
    self._checker.check_art_test_executable('runtime_test')
    self._checker.check_art_test_executable('safe_math_test')
    self._checker.check_art_test_executable('space_bitmap_test')
    self._checker.check_art_test_executable('space_create_test')
    self._checker.check_art_test_executable('stub_test')
    self._checker.check_art_test_executable('subtype_check_info_test')
    self._checker.check_art_test_executable('subtype_check_test')
    self._checker.check_art_test_executable('system_weak_test')
    self._checker.check_art_test_executable('task_processor_test')
    self._checker.check_art_test_executable('thread_pool_test')
    self._checker.check_art_test_executable('timing_logger_test')
    self._checker.check_art_test_executable('transaction_test')
    self._checker.check_art_test_executable('two_runtimes_test')
    self._checker.check_art_test_executable('unstarted_runtime_test')
    self._checker.check_art_test_executable('var_handle_test')
    self._checker.check_art_test_executable('vdex_file_test')

    # Check sigchainlib tests.
    self._checker.check_art_test_executable('sigchain_test')

    # Check ART test (internal) libraries.
    self._checker.check_native_library('libart-gtest')
    self._checker.check_native_library('libartd-simulator-container')

    # Check ART test tools.
    self._checker.check_executable('signal_dumper')


class NoSuperfluousBinariesChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'No superfluous binaries checker'

  def run(self):
    self._checker.check_no_superfluous_files('bin')


class NoSuperfluousLibrariesChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'No superfluous libraries checker'

  def run(self):
    self._checker.check_no_superfluous_files('javalib')
    self._checker.check_no_superfluous_files('lib')
    self._checker.check_no_superfluous_files('lib64')


class NoSuperfluousArtTestsChecker:
  def __init__(self, checker):
    self._checker = checker

  def __str__(self):
    return 'No superfluous ART tests checker'

  def run(self):
    for arch in ARCHS:
      self._checker.check_no_superfluous_files('%s/%s' % (ART_TEST_DIR, arch))


class List:
  def __init__(self, provider, print_size=False):
    self._provider = provider
    self._print_size = print_size

  def print_list(self):

    def print_list_rec(path):
      apex_map = self._provider.read_dir(path)
      if apex_map is None:
        return
      apex_map = dict(apex_map)
      if '.' in apex_map:
        del apex_map['.']
      if '..' in apex_map:
        del apex_map['..']
      for (_, val) in sorted(apex_map.items()):
        val_path = os.path.join(path, val.name)
        if self._print_size:
          if val.size < 0:
            print('[    n/a    ]  %s' % val_path)
          else:
            print('[%11d]  %s' % (val.size, val_path))
        else:
          print(val_path)
        if val.is_dir:
          print_list_rec(val_path)

    print_list_rec('')


class Tree:
  def __init__(self, provider, title, print_size=False):
    print('%s' % title)
    self._provider = provider
    self._has_next_list = []
    self._print_size = print_size

  @staticmethod
  def get_vertical(has_next_list):
    string = ''
    for v in has_next_list:
      string += '%s   ' % ('│' if v else ' ')
    return string

  @staticmethod
  def get_last_vertical(last):
    return '└── ' if last else '├── '

  def print_tree(self):

    def print_tree_rec(path):
      apex_map = self._provider.read_dir(path)
      if apex_map is None:
        return
      apex_map = dict(apex_map)
      if '.' in apex_map:
        del apex_map['.']
      if '..' in apex_map:
        del apex_map['..']
      key_list = list(sorted(apex_map.keys()))
      for i, key in enumerate(key_list):
        prev = self.get_vertical(self._has_next_list)
        last = self.get_last_vertical(i == len(key_list) - 1)
        val = apex_map[key]
        if self._print_size:
          if val.size < 0:
            print('%s%s[    n/a    ]  %s' % (prev, last, val.name))
          else:
            print('%s%s[%11d]  %s' % (prev, last, val.size, val.name))
        else:
          print('%s%s%s' % (prev, last, val.name))
        if val.is_dir:
          self._has_next_list.append(i < len(key_list) - 1)
          val_path = os.path.join(path, val.name)
          print_tree_rec(val_path)
          self._has_next_list.pop()

    print_tree_rec('')


# Note: do not sys.exit early, for __del__ cleanup.
def art_apex_test_main(test_args):
  if test_args.host and test_args.flattened:
    logging.error("Both of --host and --flattened set")
    return 1
  if test_args.list and test_args.tree:
    logging.error("Both of --list and --tree set")
    return 1
  if test_args.size and not (test_args.list or test_args.tree):
    logging.error("--size set but neither --list nor --tree set")
    return 1
  if not test_args.flattened and not test_args.tmpdir:
    logging.error("Need a tmpdir.")
    return 1
  if not test_args.flattened and not test_args.host and not test_args.debugfs:
    logging.error("Need debugfs.")
    return 1

  if test_args.host:
    # Host APEX.
    if test_args.flavor not in [FLAVOR_DEBUG, FLAVOR_AUTO]:
      logging.error("Using option --host with non-Debug APEX")
      return 1
    # Host APEX is always a debug flavor (for now).
    test_args.flavor = FLAVOR_DEBUG
  else:
    # Device APEX.
    if test_args.flavor == FLAVOR_AUTO:
      logging.warning('--flavor=auto, trying to autodetect. This may be incorrect!')
      for flavor in [ FLAVOR_RELEASE, FLAVOR_DEBUG, FLAVOR_TESTING ]:
        flavor_pattern = '*.%s*' % flavor
        if fnmatch.fnmatch(test_args.apex, flavor_pattern):
          test_args.flavor = flavor
          break
      if test_args.flavor == FLAVOR_AUTO:
        logging.error('  Could not detect APEX flavor, neither \'%s\', \'%s\' nor \'%s\' in \'%s\'',
                    FLAVOR_RELEASE, FLAVOR_DEBUG, FLAVOR_TESTING, test_args.apex)
        return 1

  try:
    if test_args.host:
      apex_provider = HostApexProvider(test_args.apex, test_args.tmpdir)
    else:
      if test_args.flattened:
        apex_provider = TargetFlattenedApexProvider(test_args.apex)
      else:
        apex_provider = TargetApexProvider(test_args.apex, test_args.tmpdir, test_args.debugfs)
  except (zipfile.BadZipFile, zipfile.LargeZipFile) as e:
    logging.error('Failed to create provider: %s', e)
    return 1

  if test_args.tree:
    Tree(apex_provider, test_args.apex, test_args.size).print_tree()
    return 0
  if test_args.list:
    List(apex_provider, test_args.size).print_list()
    return 0

  checkers = []
  if test_args.bitness == BITNESS_AUTO:
    logging.warning('--bitness=auto, trying to autodetect. This may be incorrect!')
    has_32 = apex_provider.get('lib') is not None
    has_64 = apex_provider.get('lib64') is not None
    if has_32 and has_64:
      logging.warning('  Detected multilib')
      test_args.bitness = BITNESS_MULTILIB
    elif has_32:
      logging.warning('  Detected 32-only')
      test_args.bitness = BITNESS_32
    elif has_64:
      logging.warning('  Detected 64-only')
      test_args.bitness = BITNESS_64
    else:
      logging.error('  Could not detect bitness, neither lib nor lib64 contained.')
      List(apex_provider).print_list()
      return 1

  if test_args.bitness == BITNESS_32:
    base_checker = Arch32Checker(apex_provider)
  elif test_args.bitness == BITNESS_64:
    base_checker = Arch64Checker(apex_provider)
  else:
    assert test_args.bitness == BITNESS_MULTILIB
    base_checker = MultilibChecker(apex_provider)

  checkers.append(ReleaseChecker(base_checker))
  if test_args.host:
    checkers.append(ReleaseHostChecker(base_checker))
  else:
    checkers.append(ReleaseTargetChecker(base_checker))
  if test_args.flavor == FLAVOR_DEBUG or test_args.flavor == FLAVOR_TESTING:
    checkers.append(DebugChecker(base_checker))
    if not test_args.host:
      checkers.append(DebugTargetChecker(base_checker))
  if test_args.flavor == FLAVOR_TESTING:
    checkers.append(TestingTargetChecker(base_checker))

  # These checkers must be last.
  checkers.append(NoSuperfluousBinariesChecker(base_checker))
  checkers.append(NoSuperfluousArtTestsChecker(base_checker))
  if not test_args.host:
    # We only care about superfluous libraries on target, where their absence
    # can be vital to ensure they get picked up from the right package.
    checkers.append(NoSuperfluousLibrariesChecker(base_checker))

  failed = False
  for checker in checkers:
    logging.info('%s...', checker)
    checker.run()
    if base_checker.error_count() > 0:
      logging.error('%s FAILED', checker)
      failed = True
    else:
      logging.info('%s SUCCEEDED', checker)
    base_checker.reset_errors()

  return 1 if failed else 0


def art_apex_test_default(test_parser):
  if 'ANDROID_PRODUCT_OUT' not in os.environ:
    logging.error('No-argument use requires ANDROID_PRODUCT_OUT')
    sys.exit(1)
  product_out = os.environ['ANDROID_PRODUCT_OUT']
  if 'ANDROID_HOST_OUT' not in os.environ:
    logging.error('No-argument use requires ANDROID_HOST_OUT')
    sys.exit(1)
  host_out = os.environ['ANDROID_HOST_OUT']

  test_args = test_parser.parse_args(['dummy'])  # For consistency.
  test_args.debugfs = '%s/bin/debugfs' % host_out
  test_args.tmpdir = '.'
  test_args.tree = False
  test_args.list = False
  test_args.bitness = BITNESS_AUTO
  failed = False

  if not os.path.exists(test_args.debugfs):
    logging.error("Cannot find debugfs (default path %s). Please build it, e.g., m debugfs",
                  test_args.debugfs)
    sys.exit(1)

  # TODO: Add host support.
  # TODO: Add support for flattened APEX packages.
  configs = [
    {'name': 'com.android.art.release', 'flavor': FLAVOR_RELEASE, 'host': False},
    {'name': 'com.android.art.debug',   'flavor': FLAVOR_DEBUG,   'host': False},
    {'name': 'com.android.art.testing', 'flavor': FLAVOR_TESTING, 'host': False},
  ]

  for config in configs:
    logging.info(config['name'])
    # TODO: Host will need different path.
    test_args.apex = '%s/system/apex/%s.apex' % (product_out, config['name'])
    if not os.path.exists(test_args.apex):
      failed = True
      logging.error("Cannot find APEX %s. Please build it first.", test_args.apex)
      continue
    test_args.flavor = config['flavor']
    test_args.host = config['host']
    failed = art_apex_test_main(test_args) != 0

  if failed:
    sys.exit(1)


if __name__ == "__main__":
  parser = argparse.ArgumentParser(description='Check integrity of an ART APEX.')

  parser.add_argument('apex', help='APEX file input')

  parser.add_argument('--host', help='Check as host APEX', action='store_true')

  parser.add_argument('--flattened', help='Check as flattened (target) APEX', action='store_true')

  parser.add_argument('--flavor', help='Check as FLAVOR APEX', choices=FLAVORS_ALL,
                      default=FLAVOR_AUTO)

  parser.add_argument('--list', help='List all files', action='store_true')
  parser.add_argument('--tree', help='Print directory tree', action='store_true')
  parser.add_argument('--size', help='Print file sizes', action='store_true')

  parser.add_argument('--tmpdir', help='Directory for temp files')
  parser.add_argument('--debugfs', help='Path to debugfs')

  parser.add_argument('--bitness', help='Bitness to check', choices=BITNESS_ALL,
                      default=BITNESS_AUTO)

  if len(sys.argv) == 1:
    art_apex_test_default(parser)
  else:
    args = parser.parse_args()

    if args is None:
      sys.exit(1)

    exit_code = art_apex_test_main(args)
    sys.exit(exit_code)



#!/usr/bin/python3
#
# Copyright (C) 2015 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Generate java test files for test 964.
"""

import os
import sys
from pathlib import Path

BUILD_TOP = os.getenv("ANDROID_BUILD_TOP")
if BUILD_TOP is None:
  print("ANDROID_BUILD_TOP not set. Please run build/envsetup.sh", file=sys.stderr)
  sys.exit(1)

# Allow us to import utils and mixins.
sys.path.append(str(Path(BUILD_TOP)/"art"/"test"/"utils"/"python"))

from testgen.utils import get_copyright, subtree_sizes, gensym, filter_blanks
import testgen.mixins as mixins

from functools import total_ordering
import itertools
import string

# The max depth the tree can have.
MAX_IFACE_DEPTH = 3

class MainClass(mixins.DumpMixin, mixins.Named, mixins.JavaFileMixin):
  """
  A Main.java file containing the Main class and the main function. It will run
  all the test functions we have.
  """

  MAIN_CLASS_TEMPLATE = """{copyright}
class Main {{
{test_groups}
{main_func}
}}
"""

  MAIN_FUNCTION_TEMPLATE = """
  public static void main(String[] args) {{
    {test_group_invoke}
  }}
"""

  TEST_GROUP_INVOKE_TEMPLATE = """
    {test_name}();
"""

  def __init__(self):
    """
    Initialize this MainClass. We start out with no tests.
    """
    self.tests = set()

  def add_test(self, ty):
    """
    Add a test for the concrete type 'ty'
    """
    self.tests.add(Func(ty))

  def get_expected(self):
    """
    Get the expected output of this test.
    """
    all_tests = sorted(self.tests)
    return filter_blanks("\n".join(a.get_expected() for a in all_tests))

  def get_name(self):
    """
    Gets the name of this class
    """
    return "Main"

  def __str__(self):
    """
    Print the java code for this test.
    """
    all_tests = sorted(self.tests)
    test_invoke = ""
    test_groups = ""
    for t in all_tests:
      test_groups += str(t)
    for t in all_tests:
      test_invoke += self.TEST_GROUP_INVOKE_TEMPLATE.format(test_name=t.get_name())
    main_func = self.MAIN_FUNCTION_TEMPLATE.format(test_group_invoke=test_invoke)

    return self.MAIN_CLASS_TEMPLATE.format(copyright = get_copyright('java'),
                                           test_groups = test_groups,
                                           main_func = main_func)

class Func(mixins.Named, mixins.NameComparableMixin):
  """
  A function that tests the functionality of a concrete type. Should only be
  constructed by MainClass.add_test.
  """

  TEST_FUNCTION_TEMPLATE = """
  public static void {fname}() {{
    try {{
      System.out.println("About to initialize {tree}");
      {farg} v = new {farg}();
      System.out.println("Initialized {tree}");
      v.touchAll();
      System.out.println("All of {tree} hierarchy initialized");
      return;
    }} catch (Error e) {{
      e.printStackTrace(System.out);
      return;
    }}
  }}
"""

  OUTPUT_FORMAT = """
About to initialize {tree}
{initialize_output}
Initialized {tree}
{touch_output}
All of {tree} hierarchy initialized
""".strip()

  def __init__(self, farg):
    """
    Initialize a test function for the given argument
    """
    self.farg = farg

  def __str__(self):
    """
    Print the java code for this test function.
    """
    return self.TEST_FUNCTION_TEMPLATE.format(fname=self.get_name(),
                                              farg=self.farg.get_name(),
                                              tree = self.farg.get_tree())

  def get_name(self):
    """
    Gets the name of this test function
    """
    return "TEST_FUNC_{}".format(self.farg.get_name())

  def get_expected(self):
    """
    Get the expected output of this function.
    """
    return self.OUTPUT_FORMAT.format(
        tree = self.farg.get_tree(),
        initialize_output = self.farg.get_initialize_output().strip(),
        touch_output = self.farg.get_touch_output().strip())

class TestClass(mixins.DumpMixin, mixins.Named, mixins.NameComparableMixin, mixins.JavaFileMixin):
  """
  A class that will be instantiated to test interface initialization order.
  """

  TEST_CLASS_TEMPLATE = """{copyright}
public class {class_name} implements {ifaces} {{
  public void marker() {{
    return;
  }}

  public void touchAll() {{
{touch_calls}
  }}
}}
"""

  TOUCH_CALL_TEMPLATE = """
    System.out.println("{class_name} touching {iface_name}");
    {iface_name}.field.touch();
"""

  TOUCH_OUTPUT_TEMPLATE = """
{class_name} touching {iface_name}
{touch_output}
""".strip()

  def __init__(self, ifaces):
    """
    Initialize this test class which implements the given interfaces
    """
    self.ifaces = ifaces
    self.class_name = "CLASS_"+gensym()

  def get_name(self):
    """
    Gets the name of this interface
    """
    return self.class_name

  def get_tree(self):
    """
    Print out a representation of the type tree of this class
    """
    return "[{fname} {iftree}]".format(fname = self.get_name(), iftree = print_tree(self.ifaces))

  def get_initialize_output(self):
    return "\n".join(map(lambda i: i.get_initialize_output().strip(), dump_tree(self.ifaces)))

  def get_touch_output(self):
    return "\n".join(map(lambda a: self.TOUCH_OUTPUT_TEMPLATE.format(
                                      class_name = self.class_name,
                                      iface_name = a.get_name(),
                                      touch_output = a.get_touch_output()).strip(),
                         self.get_all_interfaces()))

  def get_all_interfaces(self):
    """
    Returns a set of all interfaces this class transitively implements
    """
    return sorted(set(dump_tree(self.ifaces)))

  def __str__(self):
    """
    Print the java code for this class.
    """
    j_ifaces = ', '.join(map(lambda a: a.get_name(), self.ifaces))
    touches  = '\n'.join(map(lambda a: self.TOUCH_CALL_TEMPLATE.format(class_name = self.class_name,
                                                                       iface_name = a.get_name()),
                             self.get_all_interfaces()))
    return self.TEST_CLASS_TEMPLATE.format(copyright = get_copyright('java'),
                                           ifaces = j_ifaces,
                                           class_name = self.class_name,
                                           touch_calls = touches)

class TestInterface(mixins.DumpMixin, mixins.Named, mixins.NameComparableMixin, mixins.JavaFileMixin):
  """
  An interface that will be used to test default method resolution order.
  """

  TEST_INTERFACE_TEMPLATE = """{copyright}
public interface {class_name} {extends} {ifaces} {{
  public static final Displayer field = new Displayer("{tree}");
  public void marker();
{funcs}
}}
"""

  DEFAULT_FUNC_TEMPLATE = """
  public default void {class_name}_DEFAULT_FUNC() {{ return; }}
"""

  OUTPUT_TEMPLATE = "initialization of {tree}"

  def __init__(self, ifaces, default):
    """
    Initialize interface with the given super-interfaces
    """
    self.ifaces = ifaces
    self.default = default
    end = "_DEFAULT" if default else ""
    self.class_name = "INTERFACE_"+gensym()+end
    self.cloned = False
    self.initialized = False

  def clone(self):
    """
    Clones this interface, returning a new one with the same structure but
    different name.
    """
    return TestInterface(tuple(map(lambda a: a.clone(), self.ifaces)), self.default)

  def get_name(self):
    """
    Gets the name of this interface
    """
    return self.class_name

  def __iter__(self):
    """
    Performs depth-first traversal of the interface tree this interface is the
    root of. Does not filter out repeats.
    """
    for i in self.ifaces:
      yield i
      yield from i

  def get_tree(self):
    """
    Print out a representation of the type tree of this class
    """
    return "[{class_name} {iftree}]".format(class_name = self.get_name(),
                                            iftree = print_tree(self.ifaces))

  def get_initialize_output(self):
    """
    Returns the expected output upon the class that implements this interface being initialized.
    """
    if self.default and not self.initialized:
      self.initialized = True
      return self.OUTPUT_TEMPLATE.format(tree = self.get_tree())
    else:
      return ""

  def get_touch_output(self):
    """
    Returns the expected output upon this interface being touched.
    """
    if not self.default and not self.initialized:
      self.initialized = True
      return self.OUTPUT_TEMPLATE.format(tree = self.get_tree())
    else:
      return ""

  def __str__(self):
    """
    Print the java code for this interface.
    """
    j_ifaces = ', '.join(map(lambda a: a.get_name(), self.ifaces))
    if self.default:
      funcs = self.DEFAULT_FUNC_TEMPLATE.format(class_name = self.class_name)
    else:
      funcs = ""
    return self.TEST_INTERFACE_TEMPLATE.format(copyright = get_copyright('java'),
                                               extends = "extends" if len(self.ifaces) else "",
                                               ifaces = j_ifaces,
                                               funcs = funcs,
                                               tree = self.get_tree(),
                                               class_name = self.class_name)

def dump_tree(ifaces):
  """
  Yields all the interfaces transitively implemented by the set in
  reverse-depth-first order
  """
  for i in ifaces:
    yield from dump_tree(i.ifaces)
    yield i

def print_tree(ifaces):
  """
  Prints the tree for the given ifaces.
  """
  return " ".join(i.get_tree() for i in  ifaces)

def clone_all(l):
  return tuple(a.clone() for a in l)

# Cached output of subtree_sizes for speed of access.
SUBTREES = [set(tuple(l) for l in subtree_sizes(i))
            for i in range(MAX_IFACE_DEPTH + 1)]

def create_test_classes():
  """
  Yield all the test classes with the different interface trees
  """
  for num in range(1, MAX_IFACE_DEPTH + 1):
    for split in SUBTREES[num]:
      ifaces = []
      for sub in split:
        ifaces.append(list(create_interface_trees(sub)))
      for supers in itertools.product(*ifaces):
        yield TestClass(clone_all(supers))
        for i in range(len(set(dump_tree(supers)) - set(supers))):
          ns = clone_all(supers)
          selected = sorted(set(dump_tree(ns)) - set(ns))[i]
          yield TestClass(tuple([selected] + list(ns)))

def create_interface_trees(num):
  """
  Yield all the interface trees up to 'num' depth.
  """
  if num == 0:
    yield TestInterface(tuple(), False)
    yield TestInterface(tuple(), True)
    return
  for split in SUBTREES[num]:
    ifaces = []
    for sub in split:
      ifaces.append(list(create_interface_trees(sub)))
    for supers in itertools.product(*ifaces):
      yield TestInterface(clone_all(supers), False)
      yield TestInterface(clone_all(supers), True)
      # TODO Should add on some from higher up the tree.

def create_all_test_files():
  """
  Creates all the objects representing the files in this test. They just need to
  be dumped.
  """
  mc = MainClass()
  classes = {mc}
  for clazz in create_test_classes():
    classes.add(clazz)
    for i in dump_tree(clazz.ifaces):
      classes.add(i)
    mc.add_test(clazz)
  return mc, classes

def main(argv):
  java_dir = Path(argv[1])
  if not java_dir.exists() or not java_dir.is_dir():
    print("{} is not a valid java dir".format(java_dir), file=sys.stderr)
    sys.exit(1)
  expected_txt = Path(argv[2])
  mainclass, all_files = create_all_test_files()
  with expected_txt.open('w') as out:
    print(mainclass.get_expected(), file=out)
  for f in all_files:
    f.dump(java_dir)

if __name__ == '__main__':
  main(sys.argv)



# Copyright (C) 2014 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from common.immutables               import ImmutableDict
from common.testing                  import ToUnicode
from file_format.c1visualizer.parser import ParseC1visualizerStream
from file_format.c1visualizer.struct import C1visualizerFile, C1visualizerPass
from file_format.checker.parser      import ParseCheckerStream, ParseCheckerAssertion
from file_format.checker.struct      import CheckerFile, TestCase, TestAssertion
from match.file                      import MatchTestCase, MatchFailedException
from match.line                      import MatchLines

import io
import unittest

CheckerException = SystemExit

class MatchLines_Test(unittest.TestCase):

  def createTestAssertion(self, checkerString):
    checkerFile = CheckerFile("<checker-file>")
    testCase = TestCase(checkerFile, "TestMethod TestPass", 0)
    return ParseCheckerAssertion(testCase, checkerString, TestAssertion.Variant.InOrder, 0)

  def tryMatch(self, checkerString, c1String, varState={}):
    return MatchLines(self.createTestAssertion(checkerString),
                      ToUnicode(c1String),
                      ImmutableDict(varState))

  def assertMatches(self, checkerString, c1String, varState={}):
    self.assertIsNotNone(self.tryMatch(checkerString, c1String, varState))

  def assertDoesNotMatch(self, checkerString, c1String, varState={}):
    self.assertIsNone(self.tryMatch(checkerString, c1String, varState))

  def test_TextAndWhitespace(self):
    self.assertMatches("foo", "foo")
    self.assertMatches("foo", "  foo  ")
    self.assertMatches("foo", "foo bar")
    self.assertDoesNotMatch("foo", "XfooX")
    self.assertDoesNotMatch("foo", "zoo")

    self.assertMatches("foo bar", "foo   bar")
    self.assertMatches("foo bar", "abc foo bar def")
    self.assertMatches("foo bar", "foo foo bar bar")

    self.assertMatches("foo bar", "foo X bar")
    self.assertDoesNotMatch("foo bar", "foo Xbar")

  def test_Pattern(self):
    self.assertMatches("foo{{A|B}}bar", "fooAbar")
    self.assertMatches("foo{{A|B}}bar", "fooBbar")
    self.assertDoesNotMatch("foo{{A|B}}bar", "fooCbar")

  def test_VariableReference(self):
    self.assertMatches("foo<<X>>bar", "foobar", {"X": ""})
    self.assertMatches("foo<<X>>bar", "fooAbar", {"X": "A"})
    self.assertMatches("foo<<X>>bar", "fooBbar", {"X": "B"})
    self.assertDoesNotMatch("foo<<X>>bar", "foobar", {"X": "A"})
    self.assertDoesNotMatch("foo<<X>>bar", "foo bar", {"X": "A"})
    with self.assertRaises(CheckerException):
      self.tryMatch("foo<<X>>bar", "foobar", {})

  def test_VariableDefinition(self):
    self.assertMatches("foo<<X:A|B>>bar", "fooAbar")
    self.assertMatches("foo<<X:A|B>>bar", "fooBbar")
    self.assertDoesNotMatch("foo<<X:A|B>>bar", "fooCbar")

    env = self.tryMatch("foo<<X:A.*B>>bar", "fooABbar", {})
    self.assertEqual(env, {"X": "AB"})
    env = self.tryMatch("foo<<X:A.*B>>bar", "fooAxxBbar", {})
    self.assertEqual(env, {"X": "AxxB"})

    self.assertMatches("foo<<X:A|B>>bar<<X>>baz", "fooAbarAbaz")
    self.assertMatches("foo<<X:A|B>>bar<<X>>baz", "fooBbarBbaz")
    self.assertDoesNotMatch("foo<<X:A|B>>bar<<X>>baz", "fooAbarBbaz")

  def test_NoVariableRedefinition(self):
    with self.assertRaises(CheckerException):
      self.tryMatch("<<X:...>><<X>><<X:...>><<X>>", "foofoobarbar")

  def test_EnvNotChangedOnPartialMatch(self):
    env = {"Y": "foo"}
    self.assertDoesNotMatch("<<X:A>>bar", "Abaz", env)
    self.assertFalse("X" in env.keys())

  def test_VariableContentEscaped(self):
    self.assertMatches("<<X:..>>foo<<X>>", ".*foo.*")
    self.assertDoesNotMatch("<<X:..>>foo<<X>>", ".*fooAAAA")


class MatchFiles_Test(unittest.TestCase):

  def assertMatches(self, checkerString, c1String):
    checkerString = \
      """
        /// CHECK-START: MyMethod MyPass
      """ + checkerString
    c1String = \
      """
        begin_compilation
          name "MyMethod"
          method "MyMethod"
          date 1234
        end_compilation
        begin_cfg
          name "MyPass"
      """ + c1String + \
      """
        end_cfg
      """
    checkerFile = ParseCheckerStream("<test-file>", "CHECK", io.StringIO(ToUnicode(checkerString)))
    c1File = ParseC1visualizerStream("<c1-file>", io.StringIO(ToUnicode(c1String)))
    assert len(checkerFile.testCases) == 1
    assert len(c1File.passes) == 1
    MatchTestCase(checkerFile.testCases[0], c1File.passes[0])

  def assertDoesNotMatch(self, checkerString, c1String):
    with self.assertRaises(MatchFailedException):
      self.assertMatches(checkerString, c1String)

  def test_Text(self):
    self.assertMatches("/// CHECK: foo bar", "foo bar")
    self.assertDoesNotMatch("/// CHECK: foo bar", "abc def")

  def test_Pattern(self):
    self.assertMatches("/// CHECK: abc {{de.}}", "abc de#")
    self.assertDoesNotMatch("/// CHECK: abc {{de.}}", "abc d#f")

  def test_Variables(self):
    self.assertMatches(
    """
      /// CHECK: foo<<X:.>>bar
      /// CHECK: abc<<X>>def
    """,
    """
      foo0bar
      abc0def
    """)
    self.assertMatches(
    """
      /// CHECK: foo<<X:([0-9]+)>>bar
      /// CHECK: abc<<X>>def
      /// CHECK: ### <<X>> ###
    """,
    """
      foo1234bar
      abc1234def
      ### 1234 ###
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK: foo<<X:([0-9]+)>>bar
      /// CHECK: abc<<X>>def
    """,
    """
      foo1234bar
      abc1235def
    """)

  def test_WholeWordMustMatch(self):
    self.assertMatches("/// CHECK: b{{.}}r", "abc bar def")
    self.assertDoesNotMatch("/// CHECK: b{{.}}r", "abc Xbar def")
    self.assertDoesNotMatch("/// CHECK: b{{.}}r", "abc barX def")
    self.assertDoesNotMatch("/// CHECK: b{{.}}r", "abc b r def")

  def test_InOrderAssertions(self):
    self.assertMatches(
    """
      /// CHECK: foo
      /// CHECK: bar
    """,
    """
      foo
      bar
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK: foo
      /// CHECK: bar
    """,
    """
      bar
      foo
    """)

  def test_NextLineAssertions(self):
    self.assertMatches(
    """
      /// CHECK:      foo
      /// CHECK-NEXT: bar
      /// CHECK-NEXT: abc
      /// CHECK:      def
    """,
    """
      foo
      bar
      abc
      def
    """)
    self.assertMatches(
    """
      /// CHECK:      foo
      /// CHECK-NEXT: bar
      /// CHECK:      def
    """,
    """
      foo
      bar
      abc
      def
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK:      foo
      /// CHECK-NEXT: bar
    """,
    """
      foo
      abc
      bar
    """)

    self.assertDoesNotMatch(
    """
      /// CHECK:      foo
      /// CHECK-NEXT: bar
    """,
    """
      bar
      foo
      abc
    """)

  def test_DagAssertions(self):
    self.assertMatches(
    """
      /// CHECK-DAG: foo
      /// CHECK-DAG: bar
    """,
    """
      foo
      bar
    """)
    self.assertMatches(
    """
      /// CHECK-DAG: foo
      /// CHECK-DAG: bar
    """,
    """
      bar
      foo
    """)

  def test_DagAssertionsScope(self):
    self.assertMatches(
    """
      /// CHECK:     foo
      /// CHECK-DAG: abc
      /// CHECK-DAG: def
      /// CHECK:     bar
    """,
    """
      foo
      def
      abc
      bar
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK:     foo
      /// CHECK-DAG: abc
      /// CHECK-DAG: def
      /// CHECK:     bar
    """,
    """
      foo
      abc
      bar
      def
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK:     foo
      /// CHECK-DAG: abc
      /// CHECK-DAG: def
      /// CHECK:     bar
    """,
    """
      foo
      def
      bar
      abc
    """)

  def test_NotAssertions(self):
    self.assertMatches(
    """
      /// CHECK-NOT: foo
    """,
    """
      abc
      def
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK-NOT: foo
    """,
    """
      abc foo
      def
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK-NOT: foo
      /// CHECK-NOT: bar
    """,
    """
      abc
      def bar
    """)

  def test_NotAssertionsScope(self):
    self.assertMatches(
    """
      /// CHECK:     abc
      /// CHECK-NOT: foo
      /// CHECK:     def
    """,
    """
      abc
      def
    """)
    self.assertMatches(
    """
      /// CHECK:     abc
      /// CHECK-NOT: foo
      /// CHECK:     def
    """,
    """
      abc
      def
      foo
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK:     abc
      /// CHECK-NOT: foo
      /// CHECK:     def
    """,
    """
      abc
      foo
      def
    """)

  def test_LineOnlyMatchesOnce(self):
    self.assertMatches(
    """
      /// CHECK-DAG: foo
      /// CHECK-DAG: foo
    """,
    """
      foo
      abc
      foo
    """)
    self.assertDoesNotMatch(
    """
      /// CHECK-DAG: foo
      /// CHECK-DAG: foo
    """,
    """
      foo
      abc
      bar
    """)

  def test_EvalAssertions(self):
    self.assertMatches("/// CHECK-EVAL: True", "foo")
    self.assertDoesNotMatch("/// CHECK-EVAL: False", "foo")

    self.assertMatches("/// CHECK-EVAL: 1 + 2 == 3", "foo")
    self.assertDoesNotMatch("/// CHECK-EVAL: 1 + 2 == 4", "foo")

    twoVarTestCase = """
                       /// CHECK-DAG: <<X:\d+>> <<Y:\d+>>
                       /// CHECK-EVAL: <<X>> > <<Y>>
                     """
    self.assertMatches(twoVarTestCase, "42 41");
    self.assertDoesNotMatch(twoVarTestCase, "42 43")



#!/usr/bin/env python
#
# Copyright 2017, The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
./generate_cmake_lists.py --project-name <project-name> --arch <arch>

- project-name - name of the new project
- arch - arch type. make generates seperate CMakeLists files for
         each architecture. To avoid collision in targets, only one of
         them can be included in the super project.

The primary objective of this file is to generate CMakeLists files
for CLion setup.

Steps to setup CLion.
1) Open the generated CMakeList file in CLion as a project.
2) Change the project root ANDROID_BUILD_TOP.
(Also, exclude projects that you don't bother about. This will make
the indexing faster).
"""

import sys
import os
import subprocess
import argparse

def get_android_build_top():
  path_to_top = os.environ.get('ANDROID_BUILD_TOP')
  if not path_to_top:
    # nothing set. try to guess it based on the relative path of this env.py file.
    this_file_path = os.path.realpath(__file__)
    path_to_top = os.path.join(os.path.dirname(this_file_path), '../..')
    path_to_top = os.path.realpath(path_to_top)

  if not os.path.exists(os.path.join(path_to_top, 'build/envsetup.sh')):
    print path_to_top
    raise AssertionError("geneate_cmake_lists.py must be located inside an android source tree")

  return path_to_top

def main():
  # Parse arguments
  parser = argparse.ArgumentParser(description="Generate CMakeLists files for ART")
  parser.add_argument('--project-name', dest="project_name", required=True,
                      help='name of the project')
  parser.add_argument('--arch', dest="arch", required=True, help='arch')
  args = parser.parse_args()
  project_name = args.project_name
  arch = args.arch

  # Invoke make to generate CMakeFiles
  os.environ['SOONG_GEN_CMAKEFILES']='1'
  os.environ['SOONG_GEN_CMAKEFILES_DEBUG']='1'

  ANDROID_BUILD_TOP = get_android_build_top()

  subprocess.check_output('build/soong/soong_ui.bash --make-mode', shell=True, cwd=ANDROID_BUILD_TOP)

  out_art_cmakelists_dir = os.path.join(ANDROID_BUILD_TOP,
                                        'out/development/ide/clion/art')

  # Prepare a list of directories containing generated CMakeLists files for sub projects.
  cmake_sub_dirs = set()
  for root, dirs, files in os.walk(out_art_cmakelists_dir):
    for name in files:
      if name == 'CMakeLists.txt':
        if (os.path.samefile(root, out_art_cmakelists_dir)):
          continue
        if arch not in root:
          continue
        cmake_sub_dir = cmake_sub_dirs.add(root.replace(out_art_cmakelists_dir,
                                                        '.'))

  # Generate CMakeLists file.
  f = open(os.path.join(out_art_cmakelists_dir, 'CMakeLists.txt'), 'w')
  f.write('cmake_minimum_required(VERSION 3.6)\n')
  f.write('project(%s)\n' % (project_name))

  for dr in cmake_sub_dirs:
    f.write('add_subdirectory(%s)\n' % (dr))


if __name__ == '__main__':
  main()



#!/usr/bin/python3
#
# Copyright 2019, The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Run a command using multiple cores in parallel. Stop when one exits zero and save the log from
that run.
"""

import argparse
import concurrent.futures
import contextlib
import itertools
import os
import os.path
import shutil
import subprocess
import tempfile


def run_one(cmd, tmpfile):
  """Run the command and log result to tmpfile. Return both the file name and returncode."""
  with open(tmpfile, "x") as fd:
    return tmpfile, subprocess.run(cmd, stdout=fd).returncode

def main():
  parser = argparse.ArgumentParser(
      description="""Run a command using multiple cores and save non-zero exit log

      The cmd should print all output to stdout. Stderr is not captured."""
  )
  parser.add_argument("--jobs", "-j", type=int, help="max number of jobs. default 60", default=60)
  parser.add_argument("cmd", help="command to run")
  parser.add_argument("--out", type=str, help="where to put result", default="out_log")
  args = parser.parse_args()
  cnt = 0
  found_fail = False
  ids = itertools.count(0)
  with tempfile.TemporaryDirectory() as td:
    print("Temporary files in {}".format(td))
    with concurrent.futures.ProcessPoolExecutor(args.jobs) as p:
      fs = set()
      while len(fs) != 0 or not found_fail:
        if not found_fail:
          for _, idx in zip(range(args.jobs - len(fs)), ids):
            fs.add(p.submit(run_one, args.cmd, os.path.join(td, "run_log." + str(idx))))
        ws = concurrent.futures.wait(fs, return_when=concurrent.futures.FIRST_COMPLETED)
        fs = ws.not_done
        done = list(map(lambda a: a.result(), ws.done))
        cnt += len(done)
        print("\r{} runs".format(cnt), end="")
        failed = [d for d,r in done if r != 0]
        succ = [d for d,r in done if r == 0]
        for f in succ:
          os.remove(f)
        if len(failed) != 0:
          if not found_fail:
            found_fail = True
            print("\rFailed at {} runs".format(cnt))
            if len(failed) != 1:
              for f,i in zip(failed, range(len(failed))):
                shutil.copyfile(f, args.out+"."+str(i))
            else:
              shutil.copyfile(failed[0], args.out)

if __name__ == '__main__':
  main()



#!/usr/bin/env python3
#
# Copyright (C) 2016 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for bisection-search module."""

import unittest

from unittest.mock import Mock

from bisection_search import BugSearch
from bisection_search import Dex2OatWrapperTestable
from bisection_search import FatalError
from bisection_search import MANDATORY_PASSES


class BisectionTestCase(unittest.TestCase):
  """BugSearch method test case.

  Integer constants were chosen arbitrarily. They should be large enough and
  random enough to ensure binary search does nontrivial work.

  Attributes:
    _METHODS: list of strings, methods compiled by testable
    _PASSES: list of strings, passes run by testable
    _FAILING_METHOD: string, name of method which fails in some tests
    _FAILING_PASS: string, name of pass which fails in some tests
    _MANDATORY_PASS: string, name of a mandatory pass
  """
  _METHODS_COUNT = 1293
  _PASSES_COUNT = 573
  _FAILING_METHOD_IDX = 237
  _FAILING_PASS_IDX = 444
  _METHODS = ['method_{0}'.format(i) for i in range(_METHODS_COUNT)]
  _PASSES = ['pass_{0}'.format(i) for i in range(_PASSES_COUNT)]
  _FAILING_METHOD = _METHODS[_FAILING_METHOD_IDX]
  _FAILING_PASS = _PASSES[_FAILING_PASS_IDX]
  _MANDATORY_PASS = MANDATORY_PASSES[0]

  def setUp(self):
    self.testable_mock = Mock(spec=Dex2OatWrapperTestable)
    self.testable_mock.GetAllMethods.return_value = self._METHODS
    self.testable_mock.GetAllPassesForMethod.return_value = self._PASSES

  def MethodFailsForAllPasses(self, compiled_methods, run_passes=None):
    return self._FAILING_METHOD not in compiled_methods

  def MethodFailsForAPass(self, compiled_methods, run_passes=None):
    return (self._FAILING_METHOD not in compiled_methods or
            (run_passes is not None and self._FAILING_PASS not in run_passes))

  def testNeverFails(self):
    self.testable_mock.Test.return_value = True
    res = BugSearch(self.testable_mock)
    self.assertEqual(res, (None, None))

  def testAlwaysFails(self):
    self.testable_mock.Test.return_value = False
    with self.assertRaises(FatalError):
      BugSearch(self.testable_mock)

  def testAMethodFailsForAllPasses(self):
    self.testable_mock.Test.side_effect = self.MethodFailsForAllPasses
    res = BugSearch(self.testable_mock)
    self.assertEqual(res, (self._FAILING_METHOD, None))

  def testAMethodFailsForAPass(self):
    self.testable_mock.Test.side_effect = self.MethodFailsForAPass
    res = BugSearch(self.testable_mock)
    self.assertEqual(res, (self._FAILING_METHOD, self._FAILING_PASS))

  def testMandatoryPassPresent(self):
    self.testable_mock.GetAllPassesForMethod.return_value += (
        [self._MANDATORY_PASS])
    self.testable_mock.Test.side_effect = self.MethodFailsForAPass
    BugSearch(self.testable_mock)
    for (ordered_args, keyword_args) in self.testable_mock.Test.call_args_list:
      passes = None
      if 'run_passes' in keyword_args:
        passes = keyword_args['run_passes']
      if len(ordered_args) > 1:  # run_passes passed as ordered argument
        passes = ordered_args[1]
      if passes is not None:
        self.assertIn(self._MANDATORY_PASS, passes)

if __name__ == '__main__':
  unittest.main()



# Copyright 2018, The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#
# !!! Keep up-to-date with var_cache.sh
#

#
# Provide a soong-build variable query mechanism that is cached
# in the current process and any other subchild process that knows
# how to parse the exported variable:
#
# export ART_TOOLS_BUILD_VAR_CACHE="..."
#
# Of the format:
#
#   <key1>='<value1>'\n
#   <key2>='<value2>'\n
#   ...
#   <keyN>='<valueN>'
#
# Note: This is intentionally the same output format as
#     build/soong/soong_ui.bash --dumpvars-mode --vars "key1 key2 ... keyN"
#
# For example, this would be a valid var-cache:
#
# export ART_TOOLS_BUILD_VAR_CACHE="ART_APEX_JARS='core-oj core-libart'"
#
# Calling into soong repeatedly is very slow; whenever it needs to be done
# more than once, the var_cache.py or var_cache.sh script should be used instead.
#

import os
import subprocess
import sys

def get_build_var(name):
  """
  Query soong build for a variable value and return it as a string.

  Var lookup is cached, subsequent var lookups in any child process
  (that includes a var-cache is free). The var must be in 'var_list'
  to participate in the cache.

  Example:
     host_out = var_cache.get_build_var('HOST_OUT')

  Note that build vars can often have spaces in them,
  so the caller must take care to ensure space-correctness.

  Raises KeyError if the variable name is not in 'var_list'.
  """
  _populate()
  _build_dict()

  value = _var_cache_dict.get(name)
  if value is None:
    _debug(_var_cache_dict)
    raise KeyError("The variable '%s' is not in 'var_list', can't lookup" %(name))

  return value

_var_cache_dict = None
_THIS_DIR = os.path.dirname(os.path.realpath(__file__))
_TOP = os.path.join(_THIS_DIR, "../../..")
_VAR_LIST_PATH = os.path.join(_THIS_DIR, "var_list")
_SOONG_UI_SCRIPT = os.path.join(_TOP, "build/soong/soong_ui.bash")
_DEBUG = False

def _populate():
  if os.environ.get('ART_TOOLS_BUILD_VAR_CACHE'):
    return

  _debug("Varcache missing (PY)... repopulate")

  interesting_vars=[]
  with open(_VAR_LIST_PATH) as f:
    for line in f.readlines():
      line = line.strip()
      if not line or line.startswith('#'):
        continue

      _debug(line)

      interesting_vars.append(line)

  _debug("Interesting vars: ", interesting_vars)

  # Invoke soong exactly once for optimal performance.
  var_values = subprocess.check_output([
      _SOONG_UI_SCRIPT, '--dumpvars-mode', '-vars', " ".join(interesting_vars)],
      cwd=_TOP)

  # Export the ART_TOOLS_BUILD_VAR_CACHE in the same format as soong_ui.bash --dumpvars-mode.
  os.environb[b'ART_TOOLS_BUILD_VAR_CACHE'] = var_values

  _debug("Soong output: ", var_values)

def _build_dict():
  global _var_cache_dict

  if _var_cache_dict:
    return

  _debug("_var_cache_build_dict()")

  _var_cache_dict = {}

  # Parse $ART_TOOLS_BUILD_VAR_CACHE, e.g.
  #   ART_APEX_JARS='core-oj core-libart conscrypt okhttp bouncycastle apache-xml'

  for line in os.environ['ART_TOOLS_BUILD_VAR_CACHE'].splitlines():
    _debug(line)
    var_name, var_value = line.split("=")
    var_value = var_value.strip("'")

    _debug("Var name =", var_name)
    _debug("Var value =", var_value)

    _var_cache_dict[var_name] = var_value

  _debug("Num entries in dict: ", len(_var_cache_dict))

def _debug(*args):
  if _DEBUG:
    print(*args, file=sys.stderr)

# Below definitions are for interactive use only, e.g.
# python -c 'import var_cache; var_cache._dump_cache()'

def _dump_cache():
  _populate()
  print(os.environ['ART_TOOLS_BUILD_VAR_CACHE'])

#get_build_var("xyz")



# Copyright (C) 2014 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from common.logger import Logger
from common.mixins import PrintableMixin

class C1visualizerFile(PrintableMixin):

  def __init__(self, fileName):
    self.fileName = fileName
    self.passes = []

  def addPass(self, new_pass):
    self.passes.append(new_pass)

  def findPass(self, name):
    for entry in self.passes:
      if entry.name == name:
        return entry
    return None

  def __eq__(self, other):
    return isinstance(other, self.__class__) \
       and self.passes == other.passes


class C1visualizerPass(PrintableMixin):

  def __init__(self, parent, name, body, startLineNo):
    self.parent = parent
    self.name = name
    self.body = body
    self.startLineNo = startLineNo

    if not self.name:
      Logger.fail("C1visualizer pass does not have a name", self.fileName, self.startLineNo)
    if not self.body:
      Logger.fail("C1visualizer pass does not have a body", self.fileName, self.startLineNo)

    self.parent.addPass(self)

  @property
  def fileName(self):
    return self.parent.fileName

  def __eq__(self, other):
    return isinstance(other, self.__class__) \
       and self.name == other.name \
       and self.body == other.body



#!/usr/bin/env python
#
# Copyright (C) 2018 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This script looks through compiled object file (stored human readable text),
# and looks for the compile-time constants (added through custom "asm" block).
#   For example:  .ascii  ">>OBJECT_ALIGNMENT_MASK $7 $0<<"
#
# It will transform each such line to #define which is usabe in assembly code.
#   For example:  #define OBJECT_ALIGNMENT_MASK 0x7
#
# Usage: make_header.py out/soong/.intermediates/.../asm_defines.o
#

import argparse
import re
import sys

def convert(input):
  """Find all defines in the compiler generated assembly and convert them to #define pragmas"""

  asm_define_re = re.compile(r'">>(\w+) (?:\$|#)?([-0-9]+) (?:\$|#)?(0|1)<<"')
  asm_defines = asm_define_re.findall(input)
  if not asm_defines:
    raise RuntimeError("Failed to find any asm defines in the input")

  # Convert the found constants to #define pragmas.
  # In case the C++ compiler decides to reorder the AsmDefinesFor_${name} functions,
  # we don't want the order of the .h file to change from one compilation to another.
  # Sorting ensures deterministic order of the #defines.
  output = []
  for name, value, negative_value in sorted(asm_defines):
    value = int(value)
    if value < 0 and negative_value == "0":
      # Overflow - uint64_t constant was pretty printed as negative value.
      value += 2 ** 64  # Python will use arbitrary precision arithmetic.
    output.append("#define {0} {1:#x}".format(name, value))
  return "\n".join(output)

if __name__ == "__main__":
  parser = argparse.ArgumentParser()
  parser.add_argument('input', help="Object file as text")
  args = parser.parse_args()
  print(convert(open(args.input, "r").read()))



# Copyright (C) 2018 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# usage: python hprofdump.py FILE
#   Dumps a binary heap dump file to text, to facilitate debugging of heap
#   dumps and heap dump viewers.

import time
import struct
import sys

filename = sys.argv[1]
hprof = open(filename, "rb")

def readu1(hprof):
  return struct.unpack('!B', hprof.read(1))[0]

def readu2(hprof):
  return struct.unpack('!H', hprof.read(2))[0]

def readu4(hprof):
  return struct.unpack('!I', hprof.read(4))[0]

def readu8(hprof):
  return struct.unpack('!Q', hprof.read(8))[0]

def readN(n, hprof):
  if n == 1:
    return readu1(hprof)
  if n == 2:
    return readu2(hprof)
  if n == 4:
    return readu4(hprof)
  if n == 8:
    return readu8(hprof)
  raise Exception("Unsupported size of readN: %d" % n)

TY_OBJECT = 2
TY_BOOLEAN = 4
TY_CHAR = 5
TY_FLOAT = 6
TY_DOUBLE = 7
TY_BYTE = 8
TY_SHORT = 9
TY_INT = 10
TY_LONG = 11

def showty(ty):
  if ty == TY_OBJECT:
    return "Object"
  if ty == TY_BOOLEAN:
    return "boolean"
  if ty == TY_CHAR:
    return "char"
  if ty == TY_FLOAT:
    return "float"
  if ty == TY_DOUBLE:
    return "double"
  if ty == TY_BYTE:
    return "byte"
  if ty == TY_SHORT:
    return "short"
  if ty == TY_INT:
    return "int"
  if ty == TY_LONG:
    return "long"
  raise Exception("Unsupported type %d" % ty)

strs = { }
def showstr(id):
  if id in strs:
    return strs[id]
  return "STR[@%x]" % id

loaded = { }
def showloaded(serial):
  if serial in loaded:
    return showstr(loaded[serial])
  return "SERIAL[@%x]" % serial

classobjs = { }
def showclassobj(id):
  if id in classobjs:
    return "%s @%x" % (showstr(classobjs[id]), id)
  return "@%x" % id


# [u1]* An initial NULL terminate series of bytes representing the format name
# and version.
version = ""
c = hprof.read(1)
while (c != '\0'):
  version += c
  c = hprof.read(1)
print "Version: %s" % version

# [u4] size of identifiers.
idsize = readu4(hprof)
print "ID Size: %d bytes" % idsize
def readID(hprof):
  return readN(idsize, hprof)

def valsize(ty):
  if ty == TY_OBJECT:
    return idsize
  if ty == TY_BOOLEAN:
    return 1
  if ty == TY_CHAR:
    return 2
  if ty == TY_FLOAT:
    return 4
  if ty == TY_DOUBLE:
    return 8
  if ty == TY_BYTE:
    return 1
  if ty == TY_SHORT:
    return 2
  if ty == TY_INT:
    return 4
  if ty == TY_LONG:
    return 8
  raise Exception("Unsupported type %d" % ty)

def readval(ty, hprof):
  return readN(valsize(ty), hprof)

# [u4] high word of number of ms since 0:00 GMT, 1/1/70
# [u4] low word of number of ms since 0:00 GMT, 1/1/70
timestamp = (readu4(hprof) << 32) | readu4(hprof)
s, ms = divmod(timestamp, 1000)
print "Date: %s.%03d" % (time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(s)), ms)

while hprof.read(1):
  hprof.seek(-1,1)
  pos = hprof.tell()
  tag = readu1(hprof)
  time = readu4(hprof)
  length = readu4(hprof)
  if tag == 0x01:
    id = readID(hprof)
    string = hprof.read(length - idsize)
    print "%d: STRING %x %s" % (pos, id, repr(string))
    strs[id] = string
  elif tag == 0x02:
    serial = readu4(hprof)
    classobj = readID(hprof)
    stack = readu4(hprof)
    classname = readID(hprof)
    loaded[serial] = classname
    classobjs[classobj] = classname
    print "LOAD CLASS #%d %s @%x stack=@%x" % (serial, showstr(classname), classobj, stack)
  elif tag == 0x04:
    id = readID(hprof)
    method = readID(hprof)
    sig = readID(hprof)
    file = readID(hprof)
    serial = readu4(hprof)
    line = readu4(hprof);
    print "STACK FRAME %d '%s' '%s' '%s' line=%d classserial=%d" % (id, showstr(method), showstr(sig), showstr(file), line, serial)
  elif tag == 0x05:
    serial = readu4(hprof)
    print "STACK TRACE %d" % serial
    thread = readu4(hprof)
    frames = readu4(hprof)
    hprof.read(idsize * frames)
  elif tag == 0x06:
    print "ALLOC SITES"
    flags = readu2(hprof)
    cutoff_ratio = readu4(hprof)
    live_bytes = readu4(hprof)
    live_insts = readu4(hprof)
    alloc_bytes = readu8(hprof)
    alloc_insts = readu8(hprof)
    numsites = readu4(hprof)
    while numsites > 0:
      indicator = readu1(hprof)
      class_serial = readu4(hprof)
      stack = readu4(hprof)
      live_bytes = readu4(hprof)
      live_insts = readu4(hprof)
      alloc_bytes = readu4(hprof)
      alloc_insts = readu4(hprof)
      numsites -= 1
  elif tag == 0x0A:
    thread = readu4(hprof)
    object = readID(hprof)
    stack = readu4(hprof)
    name = readID(hprof)
    group_name = readID(hprof)
    pgroup_name = readID(hprof)
    print "START THREAD serial=%d" % thread
  elif tag == 0x0B:
    thread = readu4(hprof)
    print "END THREAD"
  elif tag == 0x0C or tag == 0x1C:
    if tag == 0x0C:
      print "HEAP DUMP"
    else:
      print "HEAP DUMP SEGMENT"

    while (length > 0):
      subtag = readu1(hprof) ; length -= 1
      if subtag == 0xFF:
        print " ROOT UNKNOWN"
        objid = readID(hprof) ; length -= idsize
      elif subtag == 0x01:
        print " ROOT JNI GLOBAL"
        objid = readID(hprof) ; length -= idsize
        ref = readID(hprof) ; length -= idsize
      elif subtag == 0x02:
        print " ROOT JNI LOCAL"
        objid = readID(hprof) ; length -= idsize
        thread = readu4(hprof) ; length -= 4
        frame = readu4(hprof) ; length -= 4
      elif subtag == 0x03:
        print " ROOT JAVA FRAME"
        objid = readID(hprof) ; length -= idsize
        serial = readu4(hprof) ; length -= 4
        frame = readu4(hprof) ; length -= 4
      elif subtag == 0x04:
        objid = readID(hprof) ; length -= idsize
        serial = readu4(hprof) ; length -= 4
        print " ROOT NATIVE STACK serial=%d" % serial
      elif subtag == 0x05:
        print " ROOT STICKY CLASS"
        objid = readID(hprof) ; length -= idsize
      elif subtag == 0x06:
        print " ROOT THREAD BLOCK"
        objid = readID(hprof) ; length -= idsize
        thread = readu4(hprof) ; length -= 4
      elif subtag == 0x07:
        print " ROOT MONITOR USED"
        objid = readID(hprof) ; length -= idsize
      elif subtag == 0x08:
        threadid = readID(hprof) ; length -= idsize
        serial = readu4(hprof) ; length -= 4
        stack = readu4(hprof) ; length -= 4
        print " ROOT THREAD OBJECT threadid=@%x serial=%d" % (threadid, serial)
      elif subtag == 0x20:
        print " CLASS DUMP"
        print "  class class object ID: %s" % showclassobj(readID(hprof)) ; length -= idsize
        print "  stack trace serial number: #%d" % readu4(hprof) ; length -= 4
        print "  super class object ID: @%x" % readID(hprof) ; length -= idsize
        print "  class loader object ID: @%x" % readID(hprof) ; length -= idsize
        print "  signers object ID: @%x" % readID(hprof) ; length -= idsize
        print "  protection domain object ID: @%x" % readID(hprof) ; length -= idsize
        print "  reserved: @%x" % readID(hprof) ; length -= idsize
        print "  reserved: @%x" % readID(hprof) ; length -= idsize
        print "  instance size (in bytes): %d" % readu4(hprof) ; length -= 4
        print "  constant pool:"
        poolsize = readu2(hprof) ; length -= 2
        while poolsize > 0:
          poolsize -= 1
          idx = readu2(hprof) ; length -= 2
          ty = readu1(hprof) ; length -= 1
          val = readval(ty, hprof) ; length -= valsize(ty)
          print "   %d %s 0x%x" % (idx, showty(ty), val)
        numstatic = readu2(hprof) ; length -= 2
        print "  static fields:"
        while numstatic > 0:
          numstatic -= 1
          nameid = readID(hprof) ; length -= idsize
          ty = readu1(hprof) ; length -= 1
          val = readval(ty, hprof) ; length -= valsize(ty)
          print "   %s %s 0x%x" % (showstr(nameid), showty(ty), val)
        numinst = readu2(hprof) ; length -= 2
        print "  instance fields:"
        while numinst > 0:
          numinst -= 1
          nameid = readID(hprof) ; length -= idsize
          ty = readu1(hprof) ; length -= 1
          print "   %s %s" % (showstr(nameid), showty(ty))
      elif subtag == 0x21:
        print " INSTANCE DUMP:"
        print "  object ID: @%x" % readID(hprof) ; length -= idsize
        stack = readu4(hprof) ; length -= 4
        print "  stack: %s" % stack
        print "  class object ID: %s" % showclassobj(readID(hprof)) ; length -= idsize
        datalen = readu4(hprof) ; length -= 4
        print "  %d bytes of instance data" % datalen
        data = hprof.read(datalen) ; length -= datalen
      elif subtag == 0x22:
        print " OBJECT ARRAY DUMP:"
        print "  array object ID: @%x" % readID(hprof) ; length -= idsize
        stack = readu4(hprof) ; length -= 4
        print "  stack: %s" % stack
        count = readu4(hprof) ; length -= 4
        print "  array class object ID: %s" % showclassobj(readID(hprof)) ; length -= idsize
        hprof.read(idsize * count) ; length -= (idsize * count)
      elif subtag == 0x23:
        print " PRIMITIVE ARRAY DUMP:"
        print "  array object ID: @%x" % readID(hprof) ; length -= idsize
        stack = readu4(hprof) ; length -= 4
        count = readu4(hprof) ; length -= 4
        ty = readu1(hprof) ; length -= 1
        hprof.read(valsize(ty)*count) ; length -= (valsize(ty)*count)
      elif subtag == 0x89:
        print " HPROF_ROOT_INTERNED_STRING"
        objid = readID(hprof) ; length -= idsize
      elif subtag == 0x8b:
        objid = readID(hprof) ; length -= idsize
        print " HPROF ROOT DEBUGGER @%x (at offset %d)" % (objid, hprof.tell() - (idsize + 1))
      elif subtag == 0x8d:
        objid = readID(hprof) ; length -= idsize
        print " HPROF ROOT VM INTERNAL @%x" % objid
      elif subtag == 0xfe:
        hty = readu4(hprof) ; length -= 4
        hnameid = readID(hprof) ; length -= idsize
        print " HPROF_HEAP_DUMP_INFO %s" % showstr(hnameid)
      else:
        raise Exception("TODO: subtag %x" % subtag)
  elif tag == 0x0E:
    flags = readu4(hprof)
    depth = readu2(hprof)
    print "CONTROL SETTINGS %x %d" % (flags, depth)
  elif tag == 0x2C:
    print "HEAP DUMP END"
  else:
    raise Exception("TODO: TAG %x" % tag)




# Copyright (C) 2014 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections                      import namedtuple
from common.immutables                import ImmutableDict
from common.logger                    import Logger
from file_format.c1visualizer.struct  import C1visualizerFile, C1visualizerPass
from file_format.checker.struct       import CheckerFile, TestCase, TestAssertion
from match.line                       import MatchLines, EvaluateLine

MatchScope = namedtuple("MatchScope", ["start", "end"])
MatchInfo = namedtuple("MatchInfo", ["scope", "variables"])

class MatchFailedException(Exception):
  def __init__(self, assertion, lineNo, variables):
    self.assertion = assertion
    self.lineNo = lineNo
    self.variables = variables

def splitIntoGroups(assertions):
  """ Breaks up a list of assertions, grouping instructions which should be
      tested in the same scope (consecutive DAG and NOT instructions).
   """
  splitAssertions = []
  lastVariant = None
  for assertion in assertions:
    if (assertion.variant == lastVariant and
        assertion.variant in [TestAssertion.Variant.DAG, TestAssertion.Variant.Not]):
      splitAssertions[-1].append(assertion)
    else:
      splitAssertions.append([assertion])
      lastVariant = assertion.variant
  return splitAssertions

def findMatchingLine(assertion, c1Pass, scope, variables, excludeLines=[]):
  """ Finds the first line in `c1Pass` which matches `assertion`.

  Scan only lines numbered between `scope.start` and `scope.end` and not on the
  `excludeLines` list.

  Returns the index of the `c1Pass` line matching the assertion and variables
  values after the match.

  Raises MatchFailedException if no such `c1Pass` line can be found.
  """
  for i in range(scope.start, scope.end):
    if i in excludeLines: continue
    newVariables = MatchLines(assertion, c1Pass.body[i], variables)
    if newVariables is not None:
      return MatchInfo(MatchScope(i, i), newVariables)
  raise MatchFailedException(assertion, scope.start, variables)

def matchDagGroup(assertions, c1Pass, scope, variables):
  """ Attempts to find matching `c1Pass` lines for a group of DAG assertions.

  Assertions are matched in the list order and variable values propagated. Only
  lines in `scope` are scanned and each line can only match one assertion.

  Returns the range of `c1Pass` lines covered by this group (min/max of matching
  line numbers) and the variable values after the match of the last assertion.

  Raises MatchFailedException when an assertion cannot be satisfied.
  """
  matchedLines = []
  for assertion in assertions:
    assert assertion.variant == TestAssertion.Variant.DAG
    match = findMatchingLine(assertion, c1Pass, scope, variables, matchedLines)
    variables = match.variables
    assert match.scope.start == match.scope.end
    assert match.scope.start not in matchedLines
    matchedLines.append(match.scope.start)
  return MatchInfo(MatchScope(min(matchedLines), max(matchedLines)), variables)

def testNotGroup(assertions, c1Pass, scope, variables):
  """ Verifies that none of the given NOT assertions matches a line inside
      the given `scope` of `c1Pass` lines.

  Raises MatchFailedException if an assertion matches a line in the scope.
  """
  for i in range(scope.start, scope.end):
    line = c1Pass.body[i]
    for assertion in assertions:
      assert assertion.variant == TestAssertion.Variant.Not
      if MatchLines(assertion, line, variables) is not None:
        raise MatchFailedException(assertion, i, variables)

def testEvalGroup(assertions, scope, variables):
  for assertion in assertions:
    if not EvaluateLine(assertion, variables):
      raise MatchFailedException(assertion, scope.start, variables)

def MatchTestCase(testCase, c1Pass):
  """ Runs a test case against a C1visualizer graph dump.

  Raises MatchFailedException when an assertion cannot be satisfied.
  """
  assert testCase.name == c1Pass.name

  matchFrom = 0
  variables = ImmutableDict()
  c1Length = len(c1Pass.body)

  # NOT assertions are verified retrospectively, once the scope is known.
  pendingNotAssertions = None

  # Prepare assertions by grouping those that are verified in the same scope.
  # We also add None as an EOF assertion that will set scope for NOTs.
  assertionGroups = splitIntoGroups(testCase.assertions)
  assertionGroups.append(None)

  for assertionGroup in assertionGroups:
    if assertionGroup is None:
      # EOF marker always matches the last+1 line of c1Pass.
      match = MatchInfo(MatchScope(c1Length, c1Length), None)
    elif assertionGroup[0].variant == TestAssertion.Variant.Not:
      # NOT assertions will be tested together with the next group.
      assert not pendingNotAssertions
      pendingNotAssertions = assertionGroup
      continue
    elif assertionGroup[0].variant == TestAssertion.Variant.InOrder:
      # Single in-order assertion. Find the first line that matches.
      assert len(assertionGroup) == 1
      scope = MatchScope(matchFrom, c1Length)
      match = findMatchingLine(assertionGroup[0], c1Pass, scope, variables)
    elif assertionGroup[0].variant == TestAssertion.Variant.NextLine:
      # Single next-line assertion. Test if the current line matches.
      assert len(assertionGroup) == 1
      scope = MatchScope(matchFrom, matchFrom + 1)
      match = findMatchingLine(assertionGroup[0], c1Pass, scope, variables)
    elif assertionGroup[0].variant == TestAssertion.Variant.DAG:
      # A group of DAG assertions. Match them all starting from the same point.
      scope = MatchScope(matchFrom, c1Length)
      match = matchDagGroup(assertionGroup, c1Pass, scope, variables)
    else:
      assert assertionGroup[0].variant == TestAssertion.Variant.Eval
      scope = MatchScope(matchFrom, c1Length)
      testEvalGroup(assertionGroup, scope, variables)
      continue

    if pendingNotAssertions:
      # Previous group were NOT assertions. Make sure they don't match any lines
      # in the [matchFrom, match.start) scope.
      scope = MatchScope(matchFrom, match.scope.start)
      testNotGroup(pendingNotAssertions, c1Pass, scope, variables)
      pendingNotAssertions = None

    # Update state.
    assert matchFrom <= match.scope.end
    matchFrom = match.scope.end + 1
    variables = match.variables

def MatchFiles(checkerFile, c1File, targetArch, debuggableMode):
  for testCase in checkerFile.testCases:
    if testCase.testArch not in [None, targetArch]:
      continue
    if testCase.forDebuggable != debuggableMode:
      continue

    # TODO: Currently does not handle multiple occurrences of the same group
    # name, e.g. when a pass is run multiple times. It will always try to
    # match a check group against the first output group of the same name.
    c1Pass = c1File.findPass(testCase.name)
    if c1Pass is None:
      with file(c1File.fileName) as cfgFile:
        Logger.log(''.join(cfgFile), Logger.Level.Error)
      Logger.fail("Test case not found in the CFG file",
                  testCase.fileName, testCase.startLineNo, testCase.name)

    Logger.startTest(testCase.name)
    try:
      MatchTestCase(testCase, c1Pass)
      Logger.testPassed()
    except MatchFailedException as e:
      lineNo = c1Pass.startLineNo + e.lineNo
      if e.assertion.variant == TestAssertion.Variant.Not:
        msg = "NOT assertion matched line {}"
      else:
        msg = "Assertion could not be matched starting from line {}"
      msg = msg.format(lineNo)
      with file(c1File.fileName) as cfgFile:
        Logger.log(''.join(cfgFile), Logger.Level.Error)
      Logger.testFailed(msg, e.assertion, e.variables)



#!/usr/bin/env python2
#
# Copyright (C) 2014 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from common.logger                 import Logger
from file_format.c1visualizer.test import C1visualizerParser_Test
from file_format.checker.test      import CheckerParser_PrefixTest, \
                                          CheckerParser_TestExpressionTest, \
                                          CheckerParser_FileLayoutTest, \
                                          CheckerParser_SuffixTests, \
                                          CheckerParser_EvalTests
from match.test                    import MatchLines_Test, \
                                          MatchFiles_Test

import unittest

if __name__ == '__main__':
  Logger.Verbosity = Logger.Level.NoOutput
  unittest.main(verbosity=2)



# Copyright (C) 2014 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from common.logger              import Logger
from file_format.checker.struct import TestExpression, TestAssertion

import re

def headAndTail(list):
  return list[0], list[1:]

def splitAtSeparators(expressions):
  """ Splits a list of TestExpressions at separators. """
  splitExpressions = []
  wordStart = 0
  for index, expression in enumerate(expressions):
    if expression.variant == TestExpression.Variant.Separator:
      splitExpressions.append(expressions[wordStart:index])
      wordStart = index + 1
  splitExpressions.append(expressions[wordStart:])
  return splitExpressions

def getVariable(name, variables, pos):
  if name in variables:
    return variables[name]
  else:
    Logger.testFailed("Missing definition of variable \"{}\"".format(name), pos, variables)

def setVariable(name, value, variables, pos):
  if name not in variables:
    return variables.copyWith(name, value)
  else:
    Logger.testFailed("Multiple definitions of variable \"{}\"".format(name), pos, variables)

def matchWords(checkerWord, stringWord, variables, pos):
  """ Attempts to match a list of TestExpressions against a string.
      Returns updated variable dictionary if successful and None otherwise.
  """
  for expression in checkerWord:
    # If `expression` is a variable reference, replace it with the value.
    if expression.variant == TestExpression.Variant.VarRef:
      pattern = re.escape(getVariable(expression.name, variables, pos))
    else:
      pattern = expression.text

    # Match the expression's regex pattern against the remainder of the word.
    # Note: re.match will succeed only if matched from the beginning.
    match = re.match(pattern, stringWord)
    if not match:
      return None

    # If `expression` was a variable definition, set the variable's value.
    if expression.variant == TestExpression.Variant.VarDef:
      variables = setVariable(expression.name, stringWord[:match.end()], variables, pos)

    # Move cursor by deleting the matched characters.
    stringWord = stringWord[match.end():]

  # Make sure the entire word matched, i.e. `stringWord` is empty.
  if stringWord:
    return None

  return variables

def MatchLines(checkerLine, stringLine, variables):
  """ Attempts to match a CHECK line against a string. Returns variable state
      after the match if successful and None otherwise.
  """
  assert checkerLine.variant != TestAssertion.Variant.Eval

  checkerWords = splitAtSeparators(checkerLine.expressions)
  stringWords = stringLine.split()

  while checkerWords:
    # Get the next run of TestExpressions which must match one string word.
    checkerWord, checkerWords = headAndTail(checkerWords)

    # Keep reading words until a match is found.
    wordMatched = False
    while stringWords:
      stringWord, stringWords = headAndTail(stringWords)
      newVariables = matchWords(checkerWord, stringWord, variables, checkerLine)
      if newVariables is not None:
        wordMatched = True
        variables = newVariables
        break
    if not wordMatched:
      return None

  # All TestExpressions matched. Return new variable state.
  return variables

def getEvalText(expression, variables, pos):
  if expression.variant == TestExpression.Variant.PlainText:
    return expression.text
  else:
    assert expression.variant == TestExpression.Variant.VarRef
    return getVariable(expression.name, variables, pos)

def EvaluateLine(checkerLine, variables):
  assert checkerLine.variant == TestAssertion.Variant.Eval
  eval_string = "".join(map(lambda expr: getEvalText(expr, variables, checkerLine),
                            checkerLine.expressions))
  return eval(eval_string)



# Copyright (C) 2014 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import collections
import sys

class Logger(object):

  class Level(object):
    NoOutput, Error, Info = range(3)

  class Color(object):
    Default, Blue, Gray, Purple, Red, Green = range(6)

    @staticmethod
    def terminalCode(color, out=sys.stdout):
      if not out.isatty():
        return ''
      elif color == Logger.Color.Blue:
        return '\033[94m'
      elif color == Logger.Color.Gray:
        return '\033[37m'
      elif color == Logger.Color.Purple:
        return '\033[95m'
      elif color == Logger.Color.Red:
        return '\033[91m'
      elif color == Logger.Color.Green:
        return '\033[32m'
      else:
        return '\033[0m'

  Verbosity = Level.Info

  @staticmethod
  def log(text, level=Level.Info, color=Color.Default, newLine=True, out=sys.stdout):
    if level <= Logger.Verbosity:
      text = Logger.Color.terminalCode(color, out) + text + \
             Logger.Color.terminalCode(Logger.Color.Default, out)
      if newLine:
        print(text, file=out)
      else:
        print(text, end="", file=out)
      out.flush()

  @staticmethod
  def fail(msg, file=None, line=-1, lineText=None, variables=None):
    Logger.log("error: ", Logger.Level.Error, color=Logger.Color.Red, newLine=False, out=sys.stderr)
    Logger.log(msg, Logger.Level.Error, out=sys.stderr)

    if lineText:
      loc = ""
      if file:
        loc += file + ":"
      if line > 0:
        loc += str(line) + ":"
      if loc:
        loc += " "
      Logger.log(loc, Logger.Level.Error, color=Logger.Color.Gray, newLine=False, out=sys.stderr)
      Logger.log(lineText, Logger.Level.Error, out=sys.stderr)

    if variables:
      longestName = 0
      for var in variables:
        longestName = max(longestName, len(var))

      for var in collections.OrderedDict(sorted(variables.items())):
        padding = ' ' * (longestName - len(var))
        Logger.log(var, Logger.Level.Error, color=Logger.Color.Green, newLine=False, out=sys.stderr)
        Logger.log(padding, Logger.Level.Error, newLine=False, out=sys.stderr)
        Logger.log(" = ", Logger.Level.Error, newLine=False, out=sys.stderr)
        Logger.log(variables[var], Logger.Level.Error, out=sys.stderr)

    sys.exit(1)

  @staticmethod
  def startTest(name):
    Logger.log("TEST ", color=Logger.Color.Purple, newLine=False)
    Logger.log(name + "... ", newLine=False)

  @staticmethod
  def testPassed():
    Logger.log("PASS", color=Logger.Color.Blue)

  @staticmethod
  def testFailed(msg, assertion, variables):
    Logger.log("FAIL", color=Logger.Color.Red)
    Logger.fail(msg, assertion.fileName, assertion.lineNo, assertion.originalText, variables)



# Copyright (C) 2014 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class EqualityMixin:
  """ Object equality via equality of dictionaries. """

  def __eq__(self, other):
    return isinstance(other, self.__class__) \
       and self.__dict__ == other.__dict__

class PrintableMixin:
  """ Prints object as name-dictionary pair. """

  def __repr__(self):
    return "<%s: %s>" % (type(self).__name__, str(self.__dict__))



#
# Copyright (C) 2016 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Common global variables and helper methods for the in-memory python script.
# The script starts with this file and is followed by the code generated form
# the templated snippets. Those define all the helper functions used below.

import sys, re
from cStringIO import StringIO

out = StringIO()  # File-like in-memory buffer.
handler_size_bytes = "MTERP_HANDLER_SIZE"
handler_size_bits = "MTERP_HANDLER_SIZE_LOG2"
opcode = ""
opnum = ""

def write_line(line):
  out.write(line + "\n")

def balign():
  write_line("    .balign {}".format(handler_size_bytes))

def write_opcode(num, name, write_method):
  global opnum, opcode
  opnum, opcode = str(num), name
  write_line("/* ------------------------------ */")
  balign()
  write_line(".L_{1}: /* {0:#04x} */".format(num, name))
  opcode_start()
  opcode_pre()
  write_method()
  opcode_end()
  write_line("")
  opnum, opcode = None, None

generated_helpers = {}

# This method generates a helper using the provided writer method.
# The output is temporarily redirected to in-memory buffer.
def add_helper(write_helper, name = None):
  if name == None:
    name = "mterp_" + opcode + "_helper"
  global out
  old_out = out
  out = StringIO()
  helper_start(name)
  write_helper()
  helper_end(name)
  out.seek(0)
  generated_helpers[name] = out.read()
  out = old_out
  return name

def generate(output_filename):
  out.seek(0)
  out.truncate()
  write_line("/* DO NOT EDIT: This file was generated by gen-mterp.py. */")
  header()
  entry()

  instruction_start()
  opcodes()
  balign()
  instruction_end()

  for name, helper in sorted(generated_helpers.items()):
    out.write(helper)
  helpers()

  footer()

  out.seek(0)
  # Squash consequtive empty lines.
  text = re.sub(r"(\n\n)(\n)+", r"\1", out.read())
  with open(output_filename, 'w') as output_file:
    output_file.write(text)




#!/usr/bin/env python3
#
# Copyright 2018, The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import subprocess
import sys
import shlex

def main():
  parser = argparse.ArgumentParser(
      description='Runs a command while collecting logcat in the background')
  parser.add_argument('--output', '-o',
                      type=lambda f: open(f,'w'),
                      required=True,
                      action='store',
                      help='File to store the logcat to. Will be created if not already existing')
  parser.add_argument('--logcat-invoke',
                      action='store',
                      default='adb logcat',
                      help="""Command to run to retrieve logcat data. Defaults to 'adb logcat'.
                      It will be run in the background and killed by SIGTERM when the 'command'
                      finishes.""")
  parser.add_argument('command',
                      action='store',
                      nargs=argparse.REMAINDER,
                      help='The command to run with logcat in the background.')
  args = parser.parse_args()
  if len(args.command) == 0:
    print("Must have some command to run.", file=sys.stderr)
    parser.print_help(file=sys.stderr)
    return 1
  # Send all output from logcat to the file.
  with subprocess.Popen(shlex.split(args.logcat_invoke),
                                    stdout=args.output,
                                    stderr=subprocess.STDOUT,
                                    shell=False,
                                    universal_newlines=True) as logcat_proc:
    # Let the run-test-proc inherit our stdout FDs
    with subprocess.Popen(shlex.split(args.command[0]) if len(args.command) == 1 else args.command,
                          stdout=None,
                          stderr=None,
                          shell=False) as run_test_proc:
      # Don't actually do anything. Just let the run-test proc finish.
      pass
    # Send SIGTERM to the logcat process.
    logcat_proc.kill()
  return 0

if __name__ == '__main__':
  sys.exit(main())




#!/usr/bin/env python
#
# Copyright (C) 2017 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Cleans up overlapping portions of traces provided by logcat."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import argparse
import re
import os

STACK_DIVIDER = 65 * "="


def match_to_int(match):
    """Returns trace line number matches as integers for sorting.
    Maps other matches to negative integers.
    """
    # Hard coded string are necessary since each trace must have the address
    # accessed, which is printed before trace lines.
    if match == "use-after-poison" or match == "unknown-crash":
        return -2
    elif match == "READ":
        return -1
    # Cutting off non-integer part of match
    return int(match[1:-1])


def clean_trace_if_valid(trace, stack_min_size, prune_exact):
    """Cleans trace if it meets a certain standard. Returns None otherwise."""
    # Note: Sample input may contain "unknown-crash" instead of
    # "use-after-poison"
    #
    # Sample input:
    #   trace:
    # "...ERROR: AddressSanitizer: use-after-poison on address 0x0071126a870a...
    # ...READ of size 2 at 0x0071126a870a thread T0 (droid.deskclock)
    # ...    #0 0x71281013b3  (/data/asan/system/lib64/libart.so+0x2263b3)
    # ...    #1 0x71280fe6b7  (/data/asan/system/lib64/libart.so+0x2236b7)
    # ...    #3 0x71280c22ef  (/data/asan/system/lib64/libart.so+0x1e72ef)
    # ...    #2 0x712810a84f  (/data/asan/system/lib64/libart.so+0x22f84f)"
    #
    #   stack_min_size: 2
    #   prune_exact: False
    #
    # Sample output:
    #
    # "...ERROR: AddressSanitizer: use-after-poison on address 0x0071126a870a...
    # ...READ of size 2 at 0x0071126a870a thread T0 (droid.deskclock)
    # ...    #0 0x71281013b3  (/data/asan/system/lib64/libart.so+0x2263b3)
    # ...    #1 0x71280fe6b7  (/data/asan/system/lib64/libart.so+0x2236b7)
    # "

    # Adds a newline character if not present at the end of trace
    trace = trace if trace[-1] == "\n" else trace + "\n"
    trace_line_matches = [(match_to_int(match.group()), match.start())
                          for match in re.finditer("#[0-9]+ "
                                                   "|use-after-poison"
                                                   "|unknown-crash"
                                                   "|READ", trace)
                          ]
    # Finds the first index where the line number ordering isn't in sequence or
    # returns the number of matches if it everything is in order.
    bad_line_no = next((i - 2 for i, match in enumerate(trace_line_matches)
                        if i - 2 != match[0]), len(trace_line_matches) - 2)
    # If the number ordering breaks after minimum stack size, then the trace is
    # still valid.
    if bad_line_no >= stack_min_size:
        # Added if the trace is already clean
        trace_line_matches.append((trace_line_matches[-1][0] + 1, len(trace)))
        bad_match = trace_line_matches[bad_line_no + 2]
        if prune_exact:
            bad_match = trace_line_matches[stack_min_size + 2]
        # Up to new-line that comes before bad line number
        return trace[:trace.rindex("\n", 0, bad_match[1]) + 1]
    return None


def extant_directory(path_name):
    """Checks if a path is an actual directory."""
    if not os.path.isdir(path_name):
        dir_error = "%s is not a directory" % (path_name)
        raise argparse.ArgumentTypeError(dir_error)
    return path_name


def parse_args():
    """Parses arguments passed in."""
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", action="store",
                        default="", dest="out_dir_name", type=extant_directory,
                        help="Output Directory")
    parser.add_argument("-e", action="store_true",
                        default=False, dest="check_exact",
                        help="Forces each trace to be cut to have "
                             "minimum number of lines")
    parser.add_argument("-m", action="store",
                        default=4, dest="stack_min_size", type=int,
                        help="minimum number of lines a trace should have")
    parser.add_argument("trace_file", action="store",
                        type=argparse.FileType("r"),
                        help="File only containing lines that are related to "
                             "Sanitizer traces")
    return parser.parse_args()


def main():
    """Parses arguments and cleans up traces using other functions."""
    stack_min_size = 4
    check_exact = False

    parsed_argv = parse_args()
    stack_min_size = parsed_argv.stack_min_size
    check_exact = parsed_argv.check_exact
    out_dir_name = parsed_argv.out_dir_name
    trace_file = parsed_argv.trace_file

    trace_split = trace_file.read().split(STACK_DIVIDER)
    trace_file.close()
    trace_clean_split = [clean_trace_if_valid(trace,
                                              stack_min_size,
                                              check_exact)
                         for trace in trace_split
                         ]
    trace_clean_split = [trace for trace in trace_clean_split
                         if trace is not None]
    filename = os.path.basename(trace_file.name + "_filtered")
    outfile = os.path.join(out_dir_name, filename)
    with open(outfile, "w") as output_file:
        output_file.write(STACK_DIVIDER.join(trace_clean_split))

    filter_percent = 100.0 - (float(len(trace_clean_split)) /
                              len(trace_split) * 100)
    filter_amount = len(trace_split) - len(trace_clean_split)
    print("Filtered out %d (%f%%) of %d. %d (%f%%) remain."
          % (filter_amount, filter_percent, len(trace_split),
             len(trace_split) - filter_amount, 1 - filter_percent))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# Copyright 2017, The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Build and run go/ab/git_master-art-host target

This script is executed by the android build server and must not be moved,
or changed in an otherwise backwards-incompatible manner.

Provided with a target name, the script setup the environment for
building the test target by taking config information from
from target_config.py.

See target_config.py for the configuration syntax.
"""

import argparse
import os
import pathlib
import subprocess
import sys

from target_config import target_config
import env

parser = argparse.ArgumentParser()
parser.add_argument('-j', default='1', dest='n_threads')
# either -l/--list OR build-target is required (but not both).
group = parser.add_mutually_exclusive_group(required=True)
group.add_argument('build_target', nargs='?')
group.add_argument('-l', '--list', action='store_true', help='List all possible run-build targets.')
options = parser.parse_args()

##########

if options.list:
  print("List of all known build_target: ")
  for k in sorted(target_config.keys()):
    print(" * " + k)
  # TODO: would be nice if this was the same order as the target config file.
  sys.exit(1)

if not target_config.get(options.build_target):
  sys.stderr.write("error: invalid build_target, see -l/--list.\n")
  sys.exit(1)

target = target_config[options.build_target]
n_threads = options.n_threads
custom_env = target.get('env', {})
custom_env['SOONG_ALLOW_MISSING_DEPENDENCIES'] = 'true'
print(custom_env)
os.environ.update(custom_env)

# build is just a binary/script that is directly executed to build any artifacts needed for the
# test.
if 'build' in target:
  build_command = target.get('build').format(
      ANDROID_BUILD_TOP = env.ANDROID_BUILD_TOP,
      MAKE_OPTIONS='DX=  -j{threads}'.format(threads = n_threads))
  sys.stdout.write(str(build_command) + '\n')
  sys.stdout.flush()
  if subprocess.call(build_command.split()):
    sys.exit(1)

# make runs soong/kati to build the target listed in the entry.
if 'make' in target:
  build_command = 'build/soong/soong_ui.bash --make-mode'
  build_command += ' DX='
  build_command += ' -j' + str(n_threads)
  build_command += ' ' + target.get('make')
  if env.DIST_DIR:
    build_command += ' dist'
  sys.stdout.write(str(build_command) + '\n')
  sys.stdout.flush()
  if subprocess.call(build_command.split()):
    sys.exit(1)

if 'golem' in target:
  machine_type = target.get('golem')
  # use art-opt-cc by default since it mimics the default preopt config.
  default_golem_config = 'art-opt-cc'

  os.chdir(env.ANDROID_BUILD_TOP)
  cmd =  ['art/tools/golem/build-target.sh']
  cmd += ['-j' + str(n_threads)]
  cmd += ['--showcommands']
  cmd += ['--machine-type=%s' %(machine_type)]
  cmd += ['--golem=%s' %(default_golem_config)]
  cmd += ['--tarball']
  sys.stdout.write(str(cmd) + '\n')
  sys.stdout.flush()

  if subprocess.call(cmd):
    sys.exit(1)

if 'run-test' in target:
  run_test_command = [os.path.join(env.ANDROID_BUILD_TOP,
                                   'art/test/testrunner/testrunner.py')]
  test_flags = target.get('run-test', [])
  out_dir = pathlib.PurePath(env.SOONG_OUT_DIR)
  if not out_dir.is_absolute():
    out_dir = pathlib.PurePath(env.ANDROID_BUILD_TOP).joinpath(out_dir)
  run_test_command += list(map(lambda a: a.format(SOONG_OUT_DIR=str(out_dir)), test_flags))
  # Let testrunner compute concurrency based on #cpus.
  # b/65822340
  # run_test_command += ['-j', str(n_threads)]

  # In the config assume everything will run with --host and on ART.
  # However for only [--jvm] this is undesirable, so don't pass in ART-specific flags.
  if ['--jvm'] != test_flags:
    run_test_command += ['--host']
    run_test_command += ['--dex2oat-jobs']
    run_test_command += ['4']
  if '--no-build-dependencies' not in test_flags:
    run_test_command += ['-b']
  run_test_command += ['--verbose']

  sys.stdout.write(str(run_test_command) + '\n')
  sys.stdout.flush()
  if subprocess.call(run_test_command):
    sys.exit(1)

sys.exit(0)



#!/usr/bin/python3
#
# Copyright (C) 2015 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Generate java test files for test 966.
"""

import generate_smali as base
import os
import sys
from pathlib import Path

BUILD_TOP = os.getenv("ANDROID_BUILD_TOP")
if BUILD_TOP is None:
  print("ANDROID_BUILD_TOP not set. Please run build/envsetup.sh", file=sys.stderr)
  sys.exit(1)

# Allow us to import mixins.
sys.path.append(str(Path(BUILD_TOP)/"art"/"test"/"utils"/"python"))

import testgen.mixins as mixins

class JavaConverter(mixins.DumpMixin, mixins.Named, mixins.JavaFileMixin):
  """
  A class that can convert a SmaliFile to a JavaFile.
  """
  def __init__(self, inner):
    self.inner = inner

  def get_name(self):
    return self.inner.get_name()

  def __str__(self):
    out = ""
    for line in str(self.inner).splitlines(keepends = True):
      if line.startswith("#"):
        out += line[1:]
    return out

def main(argv):
  final_java_dir = Path(argv[1])
  if not final_java_dir.exists() or not final_java_dir.is_dir():
    print("{} is not a valid java dir".format(final_java_dir), file=sys.stderr)
    sys.exit(1)
  initial_java_dir = Path(argv[2])
  if not initial_java_dir.exists() or not initial_java_dir.is_dir():
    print("{} is not a valid java dir".format(initial_java_dir), file=sys.stderr)
    sys.exit(1)
  expected_txt = Path(argv[3])
  mainclass, all_files = base.create_all_test_files()
  with expected_txt.open('w') as out:
    print(mainclass.get_expected(), file=out)
  for f in all_files:
    if f.initial_build_different():
      JavaConverter(f).dump(final_java_dir)
      JavaConverter(f.get_initial_build_version()).dump(initial_java_dir)
    else:
      JavaConverter(f).dump(initial_java_dir)
      if isinstance(f, base.TestInterface):
        JavaConverter(f).dump(final_java_dir)


if __name__ == '__main__':
  main(sys.argv)



#!/usr/bin/env python2
#
# Copyright (C) 2014 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from common.testing                  import ToUnicode
from file_format.c1visualizer.parser import ParseC1visualizerStream
from file_format.c1visualizer.struct import C1visualizerFile, C1visualizerPass

import io
import unittest

class C1visualizerParser_Test(unittest.TestCase):

  def createFile(self, passList):
    """ Creates an instance of CheckerFile from provided info.

    Data format: [ ( <case-name>, [ ( <text>, <assert-variant> ), ... ] ), ... ]
    """
    c1File = C1visualizerFile("<c1_file>")
    for passEntry in passList:
      passName = passEntry[0]
      passBody = passEntry[1]
      c1Pass = C1visualizerPass(c1File, passName, passBody, 0)
    return c1File

  def assertParsesTo(self, c1Text, expectedData):
    expectedFile = self.createFile(expectedData)
    actualFile = ParseC1visualizerStream("<c1_file>", io.StringIO(ToUnicode(c1Text)))
    return self.assertEqual(expectedFile, actualFile)

  def test_EmptyFile(self):
    self.assertParsesTo("", [])

  def test_SingleGroup(self):
    self.assertParsesTo(
      """
        begin_compilation
          method "MyMethod"
        end_compilation
        begin_cfg
          name "pass1"
          foo
          bar
        end_cfg
      """,
      [ ( "MyMethod pass1", [ "foo", "bar" ] ) ])

  def test_MultipleGroups(self):
    self.assertParsesTo(
      """
        begin_compilation
          name "xyz1"
          method "MyMethod1"
          date 1234
        end_compilation
        begin_cfg
          name "pass1"
          foo
          bar
        end_cfg
        begin_cfg
          name "pass2"
          abc
          def
        end_cfg
      """,
      [ ( "MyMethod1 pass1", [ "foo", "bar" ] ),
        ( "MyMethod1 pass2", [ "abc", "def" ] ) ])
    self.assertParsesTo(
      """
        begin_compilation
          name "xyz1"
          method "MyMethod1"
          date 1234
        end_compilation
        begin_cfg
          name "pass1"
          foo
          bar
        end_cfg
        begin_compilation
          name "xyz2"
          method "MyMethod2"
          date 5678
        end_compilation
        begin_cfg
          name "pass2"
          abc
          def
        end_cfg
      """,
      [ ( "MyMethod1 pass1", [ "foo", "bar" ] ),
        ( "MyMethod2 pass2", [ "abc", "def" ] ) ])
